
# line 2 "dgn_comp.y"
/*	SCCS Id: @(#)dgn_comp.c	3.2	94/10/08	*/
/*	Copyright (c) 1989 by Jean-Christophe Collet */
/*	Copyright (c) 1990 by M. Stephenson				  */
/* NetHack may be freely redistributed.  See license for details. */

/*
 * This file contains the Dungeon Compiler code
 */

/* In case we're using bison in AIX.  This definition must be
 * placed before any other C-language construct in the file
 * excluding comments and preprocessor directives (thanks IBM
 * for this wonderful feature...).
 *
 * Note: some cpps barf on this 'undefined control' (#pragma).
 * Addition of the leading space seems to prevent barfage for now,
 * and AIX will still see the directive in its non-standard locale.
 */

#ifdef _AIX
 #pragma alloca		/* keep leading space! */
#endif

#include "config.h"
#include "date.h"
#include "dgn_file.h"

void FDECL(yyerror, (const char *));
void FDECL(yywarning, (const char *));
int NDECL(yylex);
int NDECL(yyparse);
int FDECL(getchain, (char *));
int NDECL(check_dungeon);
int NDECL(check_branch);
int NDECL(check_level);
void NDECL(init_dungeon);
void NDECL(init_branch);
void NDECL(init_level);
void NDECL(output_dgn);

#define Free(ptr)		free((genericptr_t)ptr)

#ifdef AMIGA
# undef	printf
#ifndef	LATTICE
# define    memset(addr,val,len)    setmem(addr,len,val)
#endif
#endif

#ifdef MICRO
# undef exit
# if !defined(MSDOS) && !defined(WIN32)
extern void FDECL(exit, (int));
# endif
#endif

#define ERR		(-1)

static struct couple couple;
static struct tmpdungeon tmpdungeon[MAXDUNGEON];
static struct tmplevel tmplevel[LEV_LIMIT];
static struct tmpbranch tmpbranch[BRANCH_LIMIT];

static int in_dungeon = 0, n_dgns = -1, n_levs = -1, n_brs = -1;

extern int fatal_error;
extern const char *fname;
extern FILE *yyin, *yyout;	/* from dgn_lex.c */


# line 73 "dgn_comp.y"
typedef union
#ifdef __cplusplus
	YYSTYPE
#endif

{
	int	i;
	char*	str;
} YYSTYPE;
# define INTEGER 257
# define A_DUNGEON 258
# define BRANCH 259
# define CHBRANCH 260
# define LEVEL 261
# define RNDLEVEL 262
# define CHLEVEL 263
# define RNDCHLEVEL 264
# define UP_OR_DOWN 265
# define PROTOFILE 266
# define DESCRIPTION 267
# define DESCRIPTOR 268
# define LEVELDESC 269
# define ALIGNMENT 270
# define LEVALIGN 271
# define ENTRY 272
# define STAIR 273
# define NO_UP 274
# define NO_DOWN 275
# define PORTAL 276
# define STRING 277

#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#ifdef __cplusplus

#ifndef yyerror
#if !defined(YACC_HAS_CPLUS_LINKAGE)
  extern "C"
#endif
	void yyerror(const char *);
#endif
#ifndef yylex
#if !defined(YACC_HAS_CPLUS_LINKAGE)
  extern "C"
#endif
	int yylex(void);
#endif
#if !defined(YACC_HAS_CPLUS_LINKAGE)
  extern "C"
#endif
	int yyparse(void);

#endif

#ifdef __STDC__
#define YYCONST const
#else
#define YYCONST
#endif
#define yyclearin yychar = -1
#define yyerrok yyerrflag = 0
extern int yychar;
extern int yyerrflag;
YYSTYPE yylval;
YYSTYPE yyval;
typedef int yytabelem;
#ifndef YYMAXDEPTH
#define YYMAXDEPTH 150
#endif
#if YYMAXDEPTH > 0
int yy_yys[YYMAXDEPTH], *yys = yy_yys;
YYSTYPE yy_yyv[YYMAXDEPTH], *yyv = yy_yyv;
#else	/* user does initial allocation */
int *yys;
YYSTYPE *yyv;
#endif
static int yymaxdepth = YYMAXDEPTH;
# define YYERRCODE 256

# line 439 "dgn_comp.y"


void
init_dungeon()
{
	if(++n_dgns > MAXDUNGEON) {
	    (void) fprintf(stderr, "FATAL - Too many dungeons (limit: %d).\n",
		    MAXDUNGEON);
	    (void) fprintf(stderr, "To increase the limit edit MAXDUNGEON in global.h\n");
	    exit(EXIT_FAILURE);
	}

	in_dungeon = 1;
	tmpdungeon[n_dgns].lev.base = 0;
	tmpdungeon[n_dgns].lev.rand = 0;
	tmpdungeon[n_dgns].chance = 100;
	Strcpy(tmpdungeon[n_dgns].name, "");
	Strcpy(tmpdungeon[n_dgns].protoname, "");
	tmpdungeon[n_dgns].flags = 0;
	tmpdungeon[n_dgns].levels = 0;
	tmpdungeon[n_dgns].branches = 0;
	tmpdungeon[n_dgns].entry_lev = 0;
}

void
init_level()
{
	if(++n_levs > LEV_LIMIT) {

		yyerror("FATAL - Too many special levels defined.");
		exit(EXIT_FAILURE);
	}
	tmplevel[n_levs].lev.base = 0;
	tmplevel[n_levs].lev.rand = 0;
	tmplevel[n_levs].chance = 100;
	tmplevel[n_levs].rndlevs = 0;
	tmplevel[n_levs].flags = 0;
	Strcpy(tmplevel[n_levs].name, "");
	tmplevel[n_levs].chain = -1;
}

void
init_branch()
{
	if(++n_brs > BRANCH_LIMIT) {

		yyerror("FATAL - Too many special levels defined.");
		exit(EXIT_FAILURE);
	}
	tmpbranch[n_brs].lev.base = 0;
	tmpbranch[n_brs].lev.rand = 0;
	Strcpy(tmpbranch[n_brs].name, "");
	tmpbranch[n_brs].chain = -1;
}

int
getchain(s)
	char	*s;
{
	int i;

	if(strlen(s)) {

	    for(i = n_levs - tmpdungeon[n_dgns].levels + 1; i <= n_levs; i++)
		if(!strcmp(tmplevel[i].name, s)) return i;

	    yyerror("Can't locate the specified chain level.");
	    return(-2);
	}
	return(-1);
}

/*
 *	Consistancy checking routines:
 *
 *	- A dungeon must have a unique name.
 *	- A dungeon must have a originating "branch" command
 *	  (except, of course, for the first dungeon).
 *	- A dungeon must have a proper depth (at least (1, 0)).
 */

int
check_dungeon()
{
	int i;

	for(i = 0; i < n_dgns; i++)
	    if(!strcmp(tmpdungeon[i].name, tmpdungeon[n_dgns].name)) {
		yyerror("Duplicate dungeon name.");
		return(0);
	    }

	if(n_dgns)
	  for(i = 0; i < n_brs - tmpdungeon[n_dgns].branches; i++) {
	    if(!strcmp(tmpbranch[i].name, tmpdungeon[n_dgns].name)) break;

	    if(i >= n_brs - tmpdungeon[n_dgns].branches) {
		yyerror("Dungeon cannot be reached.");
		return(0);
	    }
	  }

	if(tmpdungeon[n_dgns].lev.base <= 0 ||
	   tmpdungeon[n_dgns].lev.rand < 0) {
		yyerror("Invalid dungeon depth specified.");
		return(0);
	}
	return(1);	/* OK */
}

/*
 *	- A level must have a unique level name.
 *	- If chained, the level used as reference for the chain
 *	  must be in this dungeon, must be previously defined, and
 *	  the level chained from must be "non-probabalistic" (ie.
 *	  have a 100% chance of existing).
 */

int
check_level()
{
	int i;

	if(!in_dungeon) {
		yyerror("Level defined outside of dungeon.");
		return(0);
	}

	for(i = 0; i < n_levs; i++)
	    if(!strcmp(tmplevel[i].name, tmplevel[n_levs].name)) {
		yyerror("Duplicate level name.");
		return(0);
	    }

	if(tmplevel[i].chain == -2) {
		yyerror("Invaild level chain reference.");
		return(0);
	} else if(tmplevel[i].chain != -1) {	/* there is a chain */
	    if(tmplevel[tmpbranch[i].chain].chance != 100) {
		yyerror("Level cannot chain from a probabalistic level.");
		return(0);
	    } else if(tmplevel[i].chain == n_levs) {
		yyerror("A level cannot chain to itself!");
		return(0);
	    }
	}
	return(1);	/* OK */
}

/*
 *	- A branch may not branch backwards - to avoid branch loops.
 *	- A branch name must be unique.
 *	  (ie. You can only have one entry point to each dungeon).
 *	- If chained, the level used as reference for the chain
 *	  must be in this dungeon, must be previously defined, and
 *	  the level chained from must be "non-probabalistic" (ie.
 *	  have a 100% chance of existing).
 */

int
check_branch()
{
	int i;

	if(!in_dungeon) {
		yyerror("Branch defined outside of dungeon.");
		return(0);
	}

	for(i = 0; i < n_dgns; i++)
	    if(!strcmp(tmpdungeon[i].name, tmpbranch[n_brs].name)) {

		yyerror("Reverse branching not allowed.");
		return(0);
	    }

	if(tmpbranch[i].chain == -2) {

		yyerror("Invaild branch chain reference.");
		return(0);
	} else if(tmpbranch[i].chain != -1) {	/* it is chained */

	    if(tmplevel[tmpbranch[i].chain].chance != 100) {
		yyerror("Branch cannot chain from a probabalistic level.");
		return(0);
	    }
	}
	return(1);	/* OK */
}

/*
 *	Output the dungon definition into a file.
 *
 *	The file will have the following format:
 *
 *	[ nethack version ID ]
 *	[ number of dungeons ]
 *	[ first dungeon struct ]
 *	[ levels for the first dungeon ]
 *	  ...
 *	[ branches for the first dungeon ]
 *	  ...
 *	[ second dungeon struct ]
 *	  ...
 */

void
output_dgn()
{
	int	nd, cl = 0, nl = 0,
		    cb = 0, nb = 0;
	static long version_info[3] = {
			VERSION_NUMBER, VERSION_FEATURES, VERSION_SANITY
	};

	if(++n_dgns <= 0) {
	    yyerror("FATAL - no dungeons were defined.");
	    exit(EXIT_FAILURE);
	}

	if (fwrite((char *)version_info, sizeof version_info, 1, yyout) != 1) {
	    yyerror("FATAL - output failure.");
	    exit(EXIT_FAILURE);
	}

	(void) fwrite((char *)&n_dgns, sizeof(int), 1, yyout);
	for (nd = 0; nd < n_dgns; nd++) {
	    (void) fwrite((char *)&tmpdungeon[nd], sizeof(struct tmpdungeon),
							1, yyout);

	    nl += tmpdungeon[nd].levels;
	    for(; cl < nl; cl++)
		(void) fwrite((char *)&tmplevel[cl], sizeof(struct tmplevel),
							1, yyout);

	    nb += tmpdungeon[nd].branches;
	    for(; cb < nb; cb++)
		(void) fwrite((char *)&tmpbranch[cb], sizeof(struct tmpbranch),
							1, yyout);
	}
	/* apparently necessary for Think C 5.x, otherwise harmless */
	(void) fflush(yyout);
}

/*dgn_comp.y*/
YYCONST yytabelem yyexca[] ={
-1, 1,
	0, -1,
	-2, 0,
	};
# define YYNPROD 49
# define YYLAST 142
YYCONST yytabelem yyact[]={

     8,    22,    23,    24,    25,    28,    29,    75,    21,    30,
    74,    26,    31,    27,    19,    60,    80,    81,    82,    83,
    92,    62,    56,    55,    52,    51,    50,    49,    48,    46,
    58,    57,    54,    53,   107,   104,   103,   102,   100,    99,
    96,    95,    84,    78,    77,    47,    91,    67,    79,    69,
    73,    72,    61,    59,    45,    44,    43,    42,    41,    40,
    39,    38,    37,    36,    35,    34,    33,    93,    90,    89,
    88,    71,   106,   105,    68,    70,     3,    13,    12,    32,
    18,    17,    16,    15,    14,    20,    11,    10,     9,     7,
     6,     5,     4,     2,     1,    76,     0,     0,     0,     0,
     0,     0,     0,     0,     0,    63,    64,     0,     0,    65,
    66,     0,     0,     0,     0,     0,     0,     0,     0,    85,
     0,     0,    86,    87,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,    94,     0,    97,    98,     0,     0,
     0,   101 };
YYCONST yytabelem yypact[]={

  -258,-10000000,  -258,-10000000,-10000000,-10000000,-10000000,-10000000,     8,-10000000,
-10000000,-10000000,-10000000,-10000000,-10000000,-10000000,-10000000,-10000000,-10000000,     7,
-10000000,     6,     5,     4,     3,     2,     1,     0,    -1,    -2,
    -3,    -4,-10000000,  -248,  -212,  -249,  -250,  -251,  -252,  -253,
  -235,  -236,  -254,  -255,  -237,  -238,  -262,-10000000,-10000000,   -12,
  -256,  -262,  -262,-10000000,-10000000,  -262,  -262,-10000000,-10000000,    34,
-10000000,    35,    28,   -13,   -14,  -267,  -270,  -213,  -214,  -257,
  -215,    34,    35,    35,    27,    26,-10000000,-10000000,    24,  -245,
-10000000,-10000000,-10000000,-10000000,    23,  -257,  -216,  -217,    34,    34,
  -218,-10000000,-10000000,  -219,  -245,-10000000,  -220,  -221,  -222,    32,
    31,-10000000,-10000000,-10000000,  -223,-10000000,-10000000,-10000000 };
YYCONST yytabelem yypgo[]={

     0,    95,    46,    48,    53,    94,    93,    76,    92,    91,
    90,    89,    47,    88,    87,    86,    85,    84,    83,    82,
    81,    80,    49,    78,    77 };
YYCONST yytabelem yyr1[]={

     0,     5,     5,     6,     6,     7,     7,     7,     7,     8,
     1,     1,     9,     9,     9,    13,    14,    16,    16,    15,
    11,    11,    11,    11,    11,    17,    17,    18,    18,    19,
    19,    20,    20,    21,    21,    10,    10,    23,    24,     3,
     3,     3,     3,     3,     2,     2,     4,    22,    12 };
YYCONST yytabelem yyr2[]={

     0,     0,     3,     2,     4,     2,     2,     2,     2,    13,
     1,     3,     2,     2,     2,     7,     2,     7,     7,     7,
     2,     2,     2,     2,     2,    13,    15,    15,    17,     7,
     7,    15,    17,    17,    19,     2,     2,    15,    17,     1,
     3,     3,     3,     3,     1,     3,     3,    11,    11 };
YYCONST yytabelem yychk[]={

-10000000,    -5,    -6,    -7,    -8,    -9,   -10,   -11,   258,   -13,
   -14,   -15,   -23,   -24,   -17,   -18,   -19,   -20,   -21,   272,
   -16,   266,   259,   260,   261,   262,   269,   271,   263,   264,
   267,   270,    -7,    58,    58,    58,    58,    58,    58,    58,
    58,    58,    58,    58,    58,    58,   277,   257,   277,   277,
   277,   277,   277,   268,   268,   277,   277,   268,   268,    -4,
   277,    64,   277,    -4,    -4,    -4,    -4,   -12,    40,   -22,
    40,    43,    64,    64,   277,   277,    -1,   257,   257,    -3,
   273,   274,   275,   276,   257,   -12,   -22,   -22,    43,    43,
    44,    -2,   265,    44,    -3,   257,   257,   -12,   -12,   257,
   257,    -2,   257,   257,   257,    41,    41,   257 };
YYCONST yytabelem yydef[]={

     1,    -2,     2,     3,     5,     6,     7,     8,     0,    12,
    13,    14,    35,    36,    20,    21,    22,    23,    24,     0,
    16,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     4,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,    15,    19,     0,
     0,     0,     0,    29,    30,     0,     0,    17,    18,     0,
    46,     0,     0,     0,     0,     0,     0,    10,     0,    39,
     0,     0,     0,     0,     0,     0,     9,    11,     0,    44,
    40,    41,    42,    43,     0,    39,    25,     0,     0,     0,
     0,    37,    45,     0,    44,    27,    26,    31,     0,     0,
     0,    38,    28,    33,    32,    48,    47,    34 };
typedef struct
#ifdef __cplusplus
	yytoktype
#endif
{ char *t_name; int t_val; } yytoktype;
#ifndef YYDEBUG
#	define YYDEBUG	0	/* don't allow debugging */
#endif

#if YYDEBUG

YYCONST yytoktype yytoks[] =
{
	"INTEGER",	257,
	"A_DUNGEON",	258,
	"BRANCH",	259,
	"CHBRANCH",	260,
	"LEVEL",	261,
	"RNDLEVEL",	262,
	"CHLEVEL",	263,
	"RNDCHLEVEL",	264,
	"UP_OR_DOWN",	265,
	"PROTOFILE",	266,
	"DESCRIPTION",	267,
	"DESCRIPTOR",	268,
	"LEVELDESC",	269,
	"ALIGNMENT",	270,
	"LEVALIGN",	271,
	"ENTRY",	272,
	"STAIR",	273,
	"NO_UP",	274,
	"NO_DOWN",	275,
	"PORTAL",	276,
	"STRING",	277,
	"-unknown-",	-1	/* ends search */
};

YYCONST char * yyreds[] =
{
	"-no such reduction-",
	"file : /* empty */",
	"file : dungeons",
	"dungeons : dungeon",
	"dungeons : dungeons dungeon",
	"dungeon : dungeonline",
	"dungeon : dungeondesc",
	"dungeon : branches",
	"dungeon : levels",
	"dungeonline : A_DUNGEON ':' STRING bones_tag rcouple optional_int",
	"optional_int : /* empty */",
	"optional_int : INTEGER",
	"dungeondesc : entry",
	"dungeondesc : descriptions",
	"dungeondesc : prototype",
	"entry : ENTRY ':' INTEGER",
	"descriptions : desc",
	"desc : DESCRIPTION ':' DESCRIPTOR",
	"desc : ALIGNMENT ':' DESCRIPTOR",
	"prototype : PROTOFILE ':' STRING",
	"levels : level1",
	"levels : level2",
	"levels : levdesc",
	"levels : chlevel1",
	"levels : chlevel2",
	"level1 : LEVEL ':' STRING bones_tag '@' acouple",
	"level1 : RNDLEVEL ':' STRING bones_tag '@' acouple INTEGER",
	"level2 : LEVEL ':' STRING bones_tag '@' acouple INTEGER",
	"level2 : RNDLEVEL ':' STRING bones_tag '@' acouple INTEGER INTEGER",
	"levdesc : LEVELDESC ':' DESCRIPTOR",
	"levdesc : LEVALIGN ':' DESCRIPTOR",
	"chlevel1 : CHLEVEL ':' STRING bones_tag STRING '+' rcouple",
	"chlevel1 : RNDCHLEVEL ':' STRING bones_tag STRING '+' rcouple INTEGER",
	"chlevel2 : CHLEVEL ':' STRING bones_tag STRING '+' rcouple INTEGER",
	"chlevel2 : RNDCHLEVEL ':' STRING bones_tag STRING '+' rcouple INTEGER INTEGER",
	"branches : branch",
	"branches : chbranch",
	"branch : BRANCH ':' STRING '@' acouple branch_type direction",
	"chbranch : CHBRANCH ':' STRING STRING '+' rcouple branch_type direction",
	"branch_type : /* empty */",
	"branch_type : STAIR",
	"branch_type : NO_UP",
	"branch_type : NO_DOWN",
	"branch_type : PORTAL",
	"direction : /* empty */",
	"direction : UP_OR_DOWN",
	"bones_tag : STRING",
	"acouple : '(' INTEGER ',' INTEGER ')'",
	"rcouple : '(' INTEGER ',' INTEGER ')'",
};
#endif /* YYDEBUG */
#ident	"@(#) yaccpar 20.1 94/12/04 "
#ident	"@(#)yacc:common/yaccpar	1.26"

/*
** Skeleton parser driver for yacc output
*/

#include <unistd.h> 	/* to get a definition for gettxt so C++ doesn't
				complain */

/*
** yacc user known macros and defines
*/
#define YYERROR		goto yyerrlab
#define YYACCEPT	return(0)
#define YYABORT		return(1)
#define YYBACKUP( newtoken, newvalue )\
{\
	if ( yychar >= 0 || ( yyr2[ yytmp ] >> 1 ) != 1 )\
	{\
		yyerror( gettxt("uxlibc:78", "Syntax error - cannot backup") );\
		goto yyerrlab;\
	}\
	yychar = newtoken;\
	yystate = *yyps;\
	yylval = newvalue;\
	goto yynewstate;\
}
#define YYRECOVERING()	(!!yyerrflag)
#define YYNEW(type)	(char *) malloc(sizeof(type) * yynewmax)
#define YYCOPY(to, from, type) \
	(type *) memcpy(to, (char *) from, yynewmax * sizeof(type))
#define YYENLARGE( from, type) \
	(type *) realloc((char *) from, yynewmax * sizeof(type))
#ifndef YYDEBUG
#	define YYDEBUG	1	/* make debugging available */
#endif

/*
** user known globals
*/
int yydebug;			/* set to 1 to get debugging */

/*
** driver internal defines
*/
#define YYFLAG		(-10000000)

/*
** global variables used by the parser
*/
YYSTYPE *yypv;			/* top of value stack */
int *yyps;			/* top of state stack */

int yystate;			/* current state */
int yytmp;			/* extra var (lasts between blocks) */

int yynerrs;			/* number of errors */
int yyerrflag;			/* error recovery flag */
int yychar;			/* current input token number */

#ifndef __HAVE_GETTXT__	/* Message catalogue stuff */
#define gettxt(a, b) b
#endif

#if defined(__cplusplus) && ! defined(YACC_HAS_CPLUS_LINKAGE)
extern "C"
#endif

/*
** yyparse - return 0 if worked, 1 if syntax error not recovered from
*/

#if defined(__STDC__) || defined(__cplusplus)
int yyparse(void)
#else
int yyparse()
#endif
{
	register YYSTYPE *yypvt;	/* top of value stack for $vars */
	int *yys_end;

	/*
	** Initialize externals - yyparse may be called more than once
	*/
	yypv = &yyv[-1];
	yyps = &yys[-1];
	yystate = 0;
	yytmp = 0;
	yynerrs = 0;
	yyerrflag = 0;
	yychar = -1;

#if YYMAXDEPTH <= 0
	if (yymaxdepth <= 0)
	{
		if ((yymaxdepth = YYEXPAND(0)) <= 0)
		{
			yyerror(gettxt("uxlibc:79", "Yacc initialization error"));
			YYABORT;
		}
	}
#endif
	yys_end = &yys[yymaxdepth];

	goto yystack;
	{
		register YYSTYPE *yy_pv;	/* top of value stack */
		register int *yy_ps;		/* top of state stack */
		register int yy_state;		/* current state */
		register int  yy_n;		/* internal state number info */

		/*
		** get globals into registers.
		** branch to here only if YYBACKUP was called.
		*/
	yynewstate:
		yy_pv = yypv;
		yy_ps = yyps;
		yy_state = yystate;
		goto yy_newstate;

		/*
		** get globals into registers.
		** either we just started, or we just finished a reduction
		*/
	yystack:
		yy_pv = yypv;
		yy_ps = yyps;
		yy_state = yystate;

		/*
		** top of for (;;) loop while no reductions done
		*/
	yy_stack:
		/*
		** put a state and value onto the stacks
		*/
#if YYDEBUG
		/*
		** if debugging, look up token value in list of value vs.
		** name pairs.  0 and negative (-1) are special values.
		** Note: linear search is used since time is not a real
		** consideration while debugging.
		*/
		if ( yydebug )
		{
			register int yy_i;

			printf( "State %d, token ", yy_state );
			if ( yychar == 0 )
				printf( "end-of-file\n" );
			else if ( yychar < 0 )
				printf( "-none-\n" );
			else
			{
				for ( yy_i = 0; yytoks[yy_i].t_val >= 0;
					yy_i++ )
				{
					if ( yytoks[yy_i].t_val == yychar )
						break;
				}
				printf( "%s\n", yytoks[yy_i].t_name );
			}
		}
#endif /* YYDEBUG */
		if ( ++yy_ps >= yys_end )	/* room on stack? */
		{
			int yynewmax;
			long byte_offset;
			int *yys_base = yys;
			YYSTYPE *yyv_base = yyv;
#ifdef YYEXPAND
			yynewmax = YYEXPAND(yymaxdepth);
#else
			yynewmax = 2 * yymaxdepth;	/* double table size */
			if (yymaxdepth == YYMAXDEPTH)	/* first time growth */
			{
				char *newyys = YYNEW(int);
				char *newyyv = YYNEW(YYSTYPE);
				if (newyys != 0 && newyyv != 0)
				{
					yys = YYCOPY(newyys, yys, int);
					yyv = YYCOPY(newyyv, yyv, YYSTYPE);
				}
				else
					yynewmax = 0;	/* failed */
			}
			else				/* not first time */
			{
				yys = YYENLARGE(yys, int);
				yyv = YYENLARGE(yyv, YYSTYPE);
				if (yys == 0 || yyv == 0)
					yynewmax = 0;	/* failed */
			}
#endif
			if (yynewmax <= yymaxdepth)	/* tables not expanded */
			{
				yyerror( gettxt("uxlibc:80", "Yacc stack overflow") );
				YYABORT;
			}
			yymaxdepth = yynewmax;
			yys_end = &yys[yymaxdepth];

			/* reset pointers into yys */
			byte_offset = (char *)yys - (char *)yys_base;
			yy_ps = (int *)(byte_offset + (char *)yy_ps);
			yyps = (int *)(byte_offset + (char *)yyps);

			/* reset pointers into yyv */
			byte_offset = (char *)yyv - (char *)yyv_base;
			yypvt = (YYSTYPE *)(byte_offset + (char *)yypvt);
			yy_pv = (YYSTYPE *)(byte_offset + (char *)yy_pv);
			yypv = (YYSTYPE *)(byte_offset + (char *)yypv);
		}
		*yy_ps = yy_state;
		*++yy_pv = yyval;

		/*
		** we have a new state - find out what to do
		*/
	yy_newstate:
		if ( ( yy_n = yypact[ yy_state ] ) <= YYFLAG )
			goto yydefault;		/* simple state */
#if YYDEBUG
		/*
		** if debugging, need to mark whether new token grabbed
		*/
		yytmp = yychar < 0;
#endif
		if ( ( yychar < 0 ) && ( ( yychar = yylex() ) < 0 ) )
			yychar = 0;		/* reached EOF */
#if YYDEBUG
		if ( yydebug && yytmp )
		{
			register int yy_i;

			printf( "Received token " );
			if ( yychar == 0 )
				printf( "end-of-file\n" );
			else if ( yychar < 0 )
				printf( "-none-\n" );
			else
			{
				for ( yy_i = 0; yytoks[yy_i].t_val >= 0;
					yy_i++ )
				{
					if ( yytoks[yy_i].t_val == yychar )
						break;
				}
				printf( "%s\n", yytoks[yy_i].t_name );
			}
		}
#endif /* YYDEBUG */
		if ( ( ( yy_n += yychar ) < 0 ) || ( yy_n >= YYLAST ) )
			goto yydefault;
		if ( yychk[ yy_n = yyact[ yy_n ] ] == yychar )	/*valid shift*/
		{
			yychar = -1;
			yyval = yylval;
			yy_state = yy_n;
			if ( yyerrflag > 0 )
				yyerrflag--;
			goto yy_stack;
		}

	yydefault:
		if ( ( yy_n = yydef[ yy_state ] ) == -2 )
		{
#if YYDEBUG
			yytmp = yychar < 0;
#endif
			if ( ( yychar < 0 ) && ( ( yychar = yylex() ) < 0 ) )
				yychar = 0;		/* reached EOF */
#if YYDEBUG
			if ( yydebug && yytmp )
			{
				register int yy_i;

				printf( "Received token " );
				if ( yychar == 0 )
					printf( "end-of-file\n" );
				else if ( yychar < 0 )
					printf( "-none-\n" );
				else
				{
					for ( yy_i = 0;
						yytoks[yy_i].t_val >= 0;
						yy_i++ )
					{
						if ( yytoks[yy_i].t_val
							== yychar )
						{
							break;
						}
					}
					printf( "%s\n", yytoks[yy_i].t_name );
				}
			}
#endif /* YYDEBUG */
			/*
			** look through exception table
			*/
			{
				register yytabelem *yyxi = (yytabelem *)yyexca;

				while ( ( *yyxi != -1 ) ||
					( yyxi[1] != yy_state ) )
				{
					yyxi += 2;
				}
				while ( ( *(yyxi += 2) >= 0 ) &&
					( *yyxi != yychar ) )
					;
				if ( ( yy_n = yyxi[1] ) < 0 )
					YYACCEPT;
			}
		}

		/*
		** check for syntax error
		*/
		if ( yy_n == 0 )	/* have an error */
		{
			/* no worry about speed here! */
			switch ( yyerrflag )
			{
			case 0:		/* new error */
				yyerror( gettxt("uxlibc:81", "Syntax error") );
				goto skip_init;
			yyerrlab:
				/*
				** get globals into registers.
				** we have a user generated syntax type error
				*/
				yy_pv = yypv;
				yy_ps = yyps;
				yy_state = yystate;
				yynerrs++;
				/* FALLTHRU */
			skip_init:
			case 1:
			case 2:		/* incompletely recovered error */
					/* try again... */
				yyerrflag = 3;
				/*
				** find state where "error" is a legal
				** shift action
				*/
				while ( yy_ps >= yys )
				{
					yy_n = yypact[ *yy_ps ] + YYERRCODE;
					if ( yy_n >= 0 && yy_n < YYLAST &&
						yychk[yyact[yy_n]] == YYERRCODE)					{
						/*
						** simulate shift of "error"
						*/
						yy_state = yyact[ yy_n ];
						goto yy_stack;
					}
					/*
					** current state has no shift on
					** "error", pop stack
					*/
#if YYDEBUG
#	define _POP_ "Error recovery pops state %d, uncovers state %d\n"
					if ( yydebug )
						printf( _POP_, *yy_ps,
							yy_ps[-1] );
#	undef _POP_
#endif
					yy_ps--;
					yy_pv--;
				}
				/*
				** there is no state on stack with "error" as
				** a valid shift.  give up.
				*/
				YYABORT;
			case 3:		/* no shift yet; eat a token */
#if YYDEBUG
				/*
				** if debugging, look up token in list of
				** pairs.  0 and negative shouldn't occur,
				** but since timing doesn't matter when
				** debugging, it doesn't hurt to leave the
				** tests here.
				*/
				if ( yydebug )
				{
					register int yy_i;

					printf( "Error recovery discards " );
					if ( yychar == 0 )
						printf( "token end-of-file\n" );
					else if ( yychar < 0 )
						printf( "token -none-\n" );
					else
					{
						for ( yy_i = 0;
							yytoks[yy_i].t_val >= 0;
							yy_i++ )
						{
							if ( yytoks[yy_i].t_val
								== yychar )
							{
								break;
							}
						}
						printf( "token %s\n",
							yytoks[yy_i].t_name );
					}
				}
#endif /* YYDEBUG */
				if ( yychar == 0 )	/* reached EOF. quit */
					YYABORT;
				yychar = -1;
				goto yy_newstate;
			}
		}/* end if ( yy_n == 0 ) */
		/*
		** reduction by production yy_n
		** put stack tops, etc. so things right after switch
		*/
#if YYDEBUG
		/*
		** if debugging, print the string that is the user's
		** specification of the reduction which is just about
		** to be done.
		*/
		if ( yydebug )
			printf( "Reduce by (%d) \"%s\"\n",
				yy_n, yyreds[ yy_n ] );
#endif
		yytmp = yy_n;			/* value to switch over */
		yypvt = yy_pv;			/* $vars top of value stack */
		/*
		** Look in goto table for next state
		** Sorry about using yy_state here as temporary
		** register variable, but why not, if it works...
		** If yyr2[ yy_n ] doesn't have the low order bit
		** set, then there is no action to be done for
		** this reduction.  So, no saving & unsaving of
		** registers done.  The only difference between the
		** code just after the if and the body of the if is
		** the goto yy_stack in the body.  This way the test
		** can be made before the choice of what to do is needed.
		*/
		{
			/* length of production doubled with extra bit */
			register int yy_len = yyr2[ yy_n ];

			if ( !( yy_len & 01 ) )
			{
				yy_len >>= 1;
				yyval = ( yy_pv -= yy_len )[1];	/* $$ = $1 */
				yy_state = yypgo[ yy_n = yyr1[ yy_n ] ] +
					*( yy_ps -= yy_len ) + 1;
				if ( yy_state >= YYLAST ||
					yychk[ yy_state =
					yyact[ yy_state ] ] != -yy_n )
				{
					yy_state = yyact[ yypgo[ yy_n ] ];
				}
				goto yy_stack;
			}
			yy_len >>= 1;
			yyval = ( yy_pv -= yy_len )[1];	/* $$ = $1 */
			yy_state = yypgo[ yy_n = yyr1[ yy_n ] ] +
				*( yy_ps -= yy_len ) + 1;
			if ( yy_state >= YYLAST ||
				yychk[ yy_state = yyact[ yy_state ] ] != -yy_n )
			{
				yy_state = yyact[ yypgo[ yy_n ] ];
			}
		}
					/* save until reenter driver code */
		yystate = yy_state;
		yyps = yy_ps;
		yypv = yy_pv;
	}
	/*
	** code supplied by user is placed in this switch
	*/
	switch( yytmp )
	{
		
case 2:
# line 90 "dgn_comp.y"
{
			output_dgn();
		  } break;
case 9:
# line 106 "dgn_comp.y"
{
			init_dungeon();
			Strcpy(tmpdungeon[n_dgns].name, yypvt[-3].str);
			tmpdungeon[n_dgns].boneschar = (char)yypvt[-2].i;
			tmpdungeon[n_dgns].lev.base = couple.base;
			tmpdungeon[n_dgns].lev.rand = couple.rand;
			tmpdungeon[n_dgns].chance = yypvt[-0].i;
			Free(yypvt[-3].str);
		  } break;
case 10:
# line 118 "dgn_comp.y"
{
			yyval.i = 0;
		  } break;
case 11:
# line 122 "dgn_comp.y"
{
			yyval.i = yypvt[-0].i;
		  } break;
case 15:
# line 133 "dgn_comp.y"
{
			tmpdungeon[n_dgns].entry_lev = yypvt[-0].i;
		  } break;
case 17:
# line 142 "dgn_comp.y"
{
			if(yypvt[-0].i <= TOWN || yypvt[-0].i >= D_ALIGN_CHAOTIC)
			    yyerror("Illegal description - ignoring!");
			else
			    tmpdungeon[n_dgns].flags |= yypvt[-0].i ;
		  } break;
case 18:
# line 149 "dgn_comp.y"
{
			if(yypvt[-0].i && yypvt[-0].i < D_ALIGN_CHAOTIC)
			    yyerror("Illegal alignment - ignoring!");
			else
			    tmpdungeon[n_dgns].flags |= yypvt[-0].i ;
		  } break;
case 19:
# line 158 "dgn_comp.y"
{
			Strcpy(tmpdungeon[n_dgns].protoname, yypvt[-0].str);
			Free(yypvt[-0].str);
		  } break;
case 25:
# line 172 "dgn_comp.y"
{
			init_level();
			Strcpy(tmplevel[n_levs].name, yypvt[-3].str);
			tmplevel[n_levs].boneschar = (char)yypvt[-2].i;
			tmplevel[n_levs].lev.base = couple.base;
			tmplevel[n_levs].lev.rand = couple.rand;
			tmpdungeon[n_dgns].levels++;
			Free(yypvt[-3].str);
		  } break;
case 26:
# line 182 "dgn_comp.y"
{
			init_level();
			Strcpy(tmplevel[n_levs].name, yypvt[-4].str);
			tmplevel[n_levs].boneschar = (char)yypvt[-3].i;
			tmplevel[n_levs].lev.base = couple.base;
			tmplevel[n_levs].lev.rand = couple.rand;
			tmplevel[n_levs].rndlevs = yypvt[-0].i;
			tmpdungeon[n_dgns].levels++;
			Free(yypvt[-4].str);
		  } break;
case 27:
# line 195 "dgn_comp.y"
{
			init_level();
			Strcpy(tmplevel[n_levs].name, yypvt[-4].str);
			tmplevel[n_levs].boneschar = (char)yypvt[-3].i;
			tmplevel[n_levs].lev.base = couple.base;
			tmplevel[n_levs].lev.rand = couple.rand;
			tmplevel[n_levs].chance = yypvt[-0].i;
			tmpdungeon[n_dgns].levels++;
			Free(yypvt[-4].str);
		  } break;
case 28:
# line 206 "dgn_comp.y"
{
			init_level();
			Strcpy(tmplevel[n_levs].name, yypvt[-5].str);
			tmplevel[n_levs].boneschar = (char)yypvt[-4].i;
			tmplevel[n_levs].lev.base = couple.base;
			tmplevel[n_levs].lev.rand = couple.rand;
			tmplevel[n_levs].chance = yypvt[-1].i;
			tmplevel[n_levs].rndlevs = yypvt[-0].i;
			tmpdungeon[n_dgns].levels++;
			Free(yypvt[-5].str);
		  } break;
case 29:
# line 220 "dgn_comp.y"
{
			if(yypvt[-0].i >= D_ALIGN_CHAOTIC)
			    yyerror("Illegal description - ignoring!");
			else
			    tmplevel[n_levs].flags |= yypvt[-0].i ;
		  } break;
case 30:
# line 227 "dgn_comp.y"
{
			if(yypvt[-0].i && yypvt[-0].i < D_ALIGN_CHAOTIC)
			    yyerror("Illegal alignment - ignoring!");
			else
			    tmplevel[n_levs].flags |= yypvt[-0].i ;
		  } break;
case 31:
# line 236 "dgn_comp.y"
{
			init_level();
			Strcpy(tmplevel[n_levs].name, yypvt[-4].str);
			tmplevel[n_levs].boneschar = (char)yypvt[-3].i;
			tmplevel[n_levs].chain = getchain(yypvt[-2].str);
			tmplevel[n_levs].lev.base = couple.base;
			tmplevel[n_levs].lev.rand = couple.rand;
			if(!check_level()) n_levs--;
			else tmpdungeon[n_dgns].levels++;
			Free(yypvt[-4].str);
			Free(yypvt[-2].str);
		  } break;
case 32:
# line 249 "dgn_comp.y"
{
			init_level();
			Strcpy(tmplevel[n_levs].name, yypvt[-5].str);
			tmplevel[n_levs].boneschar = (char)yypvt[-4].i;
			tmplevel[n_levs].chain = getchain(yypvt[-3].str);
			tmplevel[n_levs].lev.base = couple.base;
			tmplevel[n_levs].lev.rand = couple.rand;
			tmplevel[n_levs].rndlevs = yypvt[-0].i;
			if(!check_level()) n_levs--;
			else tmpdungeon[n_dgns].levels++;
			Free(yypvt[-5].str);
			Free(yypvt[-3].str);
		  } break;
case 33:
# line 265 "dgn_comp.y"
{
			init_level();
			Strcpy(tmplevel[n_levs].name, yypvt[-5].str);
			tmplevel[n_levs].boneschar = (char)yypvt[-4].i;
			tmplevel[n_levs].chain = getchain(yypvt[-3].str);
			tmplevel[n_levs].lev.base = couple.base;
			tmplevel[n_levs].lev.rand = couple.rand;
			tmplevel[n_levs].chance = yypvt[-0].i;
			if(!check_level()) n_levs--;
			else tmpdungeon[n_dgns].levels++;
			Free(yypvt[-5].str);
			Free(yypvt[-3].str);
		  } break;
case 34:
# line 279 "dgn_comp.y"
{
			init_level();
			Strcpy(tmplevel[n_levs].name, yypvt[-6].str);
			tmplevel[n_levs].boneschar = (char)yypvt[-5].i;
			tmplevel[n_levs].chain = getchain(yypvt[-4].str);
			tmplevel[n_levs].lev.base = couple.base;
			tmplevel[n_levs].lev.rand = couple.rand;
			tmplevel[n_levs].chance = yypvt[-1].i;
			tmplevel[n_levs].rndlevs = yypvt[-0].i;
			if(!check_level()) n_levs--;
			else tmpdungeon[n_dgns].levels++;
			Free(yypvt[-6].str);
			Free(yypvt[-4].str);
		  } break;
case 37:
# line 300 "dgn_comp.y"
{
			init_branch();
			Strcpy(tmpbranch[n_brs].name, yypvt[-4].str);
			tmpbranch[n_brs].lev.base = couple.base;
			tmpbranch[n_brs].lev.rand = couple.rand;
			tmpbranch[n_brs].type = yypvt[-1].i;
			tmpbranch[n_brs].up = yypvt[-0].i;
			if(!check_branch()) n_brs--;
			else tmpdungeon[n_dgns].branches++;
			Free(yypvt[-4].str);
		  } break;
case 38:
# line 314 "dgn_comp.y"
{
			init_branch();
			Strcpy(tmpbranch[n_brs].name, yypvt[-5].str);
			tmpbranch[n_brs].chain = getchain(yypvt[-4].str);
			tmpbranch[n_brs].lev.base = couple.base;
			tmpbranch[n_brs].lev.rand = couple.rand;
			tmpbranch[n_brs].type = yypvt[-1].i;
			tmpbranch[n_brs].up = yypvt[-0].i;
			if(!check_branch()) n_brs--;
			else tmpdungeon[n_dgns].branches++;
			Free(yypvt[-5].str);
			Free(yypvt[-4].str);
		  } break;
case 39:
# line 330 "dgn_comp.y"
{
			yyval.i = TBR_STAIR;	/* two way stair */
		  } break;
case 40:
# line 334 "dgn_comp.y"
{
			yyval.i = TBR_STAIR;	/* two way stair */
		  } break;
case 41:
# line 338 "dgn_comp.y"
{
			yyval.i = TBR_NO_UP;	/* no up staircase */
		  } break;
case 42:
# line 342 "dgn_comp.y"
{
			yyval.i = TBR_NO_DOWN;	/* no down staircase */
		  } break;
case 43:
# line 346 "dgn_comp.y"
{
			yyval.i = TBR_PORTAL;	/* portal connection */
		  } break;
case 44:
# line 352 "dgn_comp.y"
{
			yyval.i = 0;	/* defaults to down */
		  } break;
case 45:
# line 356 "dgn_comp.y"
{
			yyval.i = yypvt[-0].i;
		  } break;
case 46:
# line 362 "dgn_comp.y"
{
			char *p = yypvt[-0].str;
			if (strlen(p) != 1) {
			    if (strcmp(p, "none") != 0)
		   yyerror("Bones marker must be a single char, or \"none\"!");
			    *p = '\0';
			}
			yyval.i = *p;
			Free(p);
		  } break;
case 47:
# line 392 "dgn_comp.y"
{
			if (yypvt[-3].i < -MAXLEVEL || yypvt[-3].i > MAXLEVEL) {
			    yyerror("Abs base out of dlevel range - zeroing!");
			    couple.base = couple.rand = 0;
			} else if (yypvt[-1].i < -1 ||
				((yypvt[-3].i < 0) ? (MAXLEVEL + yypvt[-3].i + yypvt[-1].i + 1) > MAXLEVEL :
					(yypvt[-3].i + yypvt[-1].i) > MAXLEVEL)) {
			    yyerror("Abs range out of dlevel range - zeroing!");
			    couple.base = couple.rand = 0;
			} else {
			    couple.base = yypvt[-3].i;
			    couple.rand = yypvt[-1].i;
			}
		  } break;
case 48:
# line 429 "dgn_comp.y"
{
			if (yypvt[-3].i < -MAXLEVEL || yypvt[-3].i > MAXLEVEL) {
			    yyerror("Rel base out of dlevel range - zeroing!");
			    couple.base = couple.rand = 0;
			} else {
			    couple.base = yypvt[-3].i;
			    couple.rand = yypvt[-1].i;
			}
		  } break;
	}
	goto yystack;		/* reset registers in driver code */
}

#undef YYCONST


/*
 * In SCO C++, crt1.o calls _main() (in libC.a) which calls main().
 * If we define a main() in liby.a, it will not get extracted, since
 * liby.a appears before libC.a on the link line.  To get around this
 * the linker must know we need main() when it is processing liby.a
 * So, we have a dummy call right here.
*/
#if defined(__cplusplus)
extern "C" int main(void);
void _yydumMain_() { main(); }
#endif

