
# line 2 "lev_comp.y"
/*	SCCS Id: @(#)lev_yacc.c	3.2	95/11/10	*/
/*	Copyright (c) 1989 by Jean-Christophe Collet */
/* NetHack may be freely redistributed.  See license for details. */

/*
 * This file contains the Level Compiler code
 * It may handle special mazes & special room-levels
 */

/* In case we're using bison in AIX.  This definition must be
 * placed before any other C-language construct in the file
 * excluding comments and preprocessor directives (thanks IBM
 * for this wonderful feature...).
 *
 * Note: some cpps barf on this 'undefined control' (#pragma).
 * Addition of the leading space seems to prevent barfage for now,
 * and AIX will still see the directive.
 */
#ifdef _AIX
 #pragma alloca		/* keep leading space! */
#endif

#include "hack.h"
#include "sp_lev.h"

#define MAX_REGISTERS	10
#define ERR		(-1)
/* many types of things are put in chars for transference to NetHack.
 * since some systems will use signed chars, limit everybody to the
 * same number for portability.
 */
#define MAX_OF_TYPE	128

#define New(type)		(type *) alloc(sizeof(type))
#define NewTab(type, size)	(type **) alloc(sizeof(type *) * size)
#define Free(ptr)		free((genericptr_t)ptr)

#ifdef MICRO
# undef exit
# if !defined(MSDOS) && !defined(WIN32)
extern void FDECL(exit, (int));
# endif
#endif

extern void FDECL(yyerror, (const char *));
extern void FDECL(yywarning, (const char *));
extern int NDECL(yylex);
int NDECL(yyparse);

extern int FDECL(get_floor_type, (CHAR_P));
extern int FDECL(get_room_type, (char *));
extern int FDECL(get_trap_type, (char *));
extern int FDECL(get_monster_id, (char *,CHAR_P));
extern int FDECL(get_object_id, (char *));
extern boolean FDECL(check_monster_char, (CHAR_P));
extern boolean FDECL(check_object_char, (CHAR_P));
extern char FDECL(what_map_char, (CHAR_P));
extern void FDECL(scan_map, (char *));
extern void NDECL(wallify_map);
extern boolean NDECL(check_subrooms);
extern void FDECL(check_coord, (int,int,const char *));
extern void NDECL(store_part);
extern void NDECL(store_room);
extern boolean FDECL(write_level_file, (char *,splev *,specialmaze *));
extern void FDECL(free_rooms, (splev *));

static struct reg {
	int x1, y1;
	int x2, y2;
}		current_region;

static struct coord {
	int x;
	int y;
}		current_coord, current_align;

static struct size {
	int height;
	int width;
}		current_size;

char tmpmessage[256];
digpos *tmppass[32];
char *tmpmap[ROWNO];

digpos *tmpdig[MAX_OF_TYPE];
region *tmpreg[MAX_OF_TYPE];
lev_region *tmplreg[MAX_OF_TYPE];
door *tmpdoor[MAX_OF_TYPE];
drawbridge *tmpdb[MAX_OF_TYPE];
walk *tmpwalk[MAX_OF_TYPE];

room_door *tmprdoor[MAX_OF_TYPE];
trap *tmptrap[MAX_OF_TYPE];
monster *tmpmonst[MAX_OF_TYPE];
object *tmpobj[MAX_OF_TYPE];
altar *tmpaltar[MAX_OF_TYPE];
lad *tmplad[MAX_OF_TYPE];
stair *tmpstair[MAX_OF_TYPE];
gold *tmpgold[MAX_OF_TYPE];
engraving *tmpengraving[MAX_OF_TYPE];
fountain *tmpfountain[MAX_OF_TYPE];
sink *tmpsink[MAX_OF_TYPE];
pool *tmppool[MAX_OF_TYPE];

mazepart *tmppart[10];
room *tmproom[MAXNROFROOMS*2];
corridor *tmpcor[MAX_OF_TYPE];

static specialmaze maze;
static splev special_lev;
static lev_init init_lev;

static char olist[MAX_REGISTERS], mlist[MAX_REGISTERS];
static struct coord plist[MAX_REGISTERS];

int n_olist = 0, n_mlist = 0, n_plist = 0;

unsigned int nlreg = 0, nreg = 0, ndoor = 0, ntrap = 0, nmons = 0, nobj = 0;
unsigned int ndb = 0, nwalk = 0, npart = 0, ndig = 0, nlad = 0, nstair = 0;
unsigned int naltar = 0, ncorridor = 0, nrooms = 0, ngold = 0, nengraving = 0;
unsigned int nfountain = 0, npool = 0, nsink = 0, npass = 0;

static int lev_flags = 0;

unsigned int max_x_map, max_y_map;

static xchar in_room;

extern int fatal_error;
extern int want_warnings;
extern const char *fname;


# line 137 "lev_comp.y"
typedef union
#ifdef __cplusplus
	YYSTYPE
#endif

{
	int	i;
	char*	map;
	struct {
		xchar room;
		xchar wall;
		xchar door;
	} corpos;
} YYSTYPE;
# define CHAR 257
# define INTEGER 258
# define BOOLEAN 259
# define MESSAGE_ID 260
# define MAZE_ID 261
# define LEVEL_ID 262
# define LEV_INIT_ID 263
# define GEOMETRY_ID 264
# define NOMAP_ID 265
# define OBJECT_ID 266
# define COBJECT_ID 267
# define MONSTER_ID 268
# define TRAP_ID 269
# define DOOR_ID 270
# define DRAWBRIDGE_ID 271
# define MAZEWALK_ID 272
# define WALLIFY_ID 273
# define REGION_ID 274
# define FILLING 275
# define RANDOM_OBJECTS_ID 276
# define RANDOM_MONSTERS_ID 277
# define RANDOM_PLACES_ID 278
# define ALTAR_ID 279
# define LADDER_ID 280
# define STAIR_ID 281
# define NON_DIGGABLE_ID 282
# define NON_PASSWALL_ID 283
# define ROOM_ID 284
# define PORTAL_ID 285
# define TELEPRT_ID 286
# define BRANCH_ID 287
# define LEV 288
# define CHANCE_ID 289
# define CORRIDOR_ID 290
# define GOLD_ID 291
# define ENGRAVING_ID 292
# define FOUNTAIN_ID 293
# define POOL_ID 294
# define SINK_ID 295
# define NONE 296
# define RAND_CORRIDOR_ID 297
# define DOOR_STATE 298
# define LIGHT_STATE 299
# define CURSE_TYPE 300
# define ENGRAVING_TYPE 301
# define DIRECTION 302
# define RANDOM_TYPE 303
# define O_REGISTER 304
# define M_REGISTER 305
# define P_REGISTER 306
# define A_REGISTER 307
# define ALIGNMENT 308
# define LEFT_OR_RIGHT 309
# define CENTER 310
# define TOP_OR_BOT 311
# define ALTAR_TYPE 312
# define UP_OR_DOWN 313
# define SUBROOM_ID 314
# define NAME_ID 315
# define FLAGS_ID 316
# define FLAG_TYPE 317
# define MON_ATTITUDE 318
# define MON_ALERTNESS 319
# define MON_APPEARANCE 320
# define CONTAINED 321
# define STRING 322
# define MAP_ID 323

#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#ifdef __cplusplus

#ifndef yyerror
#if !defined(YACC_HAS_CPLUS_LINKAGE)
  extern "C"
#endif
	void yyerror(const char *);
#endif
#ifndef yylex
#if !defined(YACC_HAS_CPLUS_LINKAGE)
  extern "C"
#endif
	int yylex(void);
#endif
#if !defined(YACC_HAS_CPLUS_LINKAGE)
  extern "C"
#endif
	int yyparse(void);

#endif

#ifdef __STDC__
#define YYCONST const
#else
#define YYCONST
#endif
#define yyclearin yychar = -1
#define yyerrok yyerrflag = 0
extern int yychar;
extern int yyerrflag;
YYSTYPE yylval;
YYSTYPE yyval;
typedef int yytabelem;
#ifndef YYMAXDEPTH
#define YYMAXDEPTH 150
#endif
#if YYMAXDEPTH > 0
int yy_yys[YYMAXDEPTH], *yys = yy_yys;
YYSTYPE yy_yyv[YYMAXDEPTH], *yyv = yy_yyv;
#else	/* user does initial allocation */
int *yys;
YYSTYPE *yyv;
#endif
static int yymaxdepth = YYMAXDEPTH;
# define YYERRCODE 256

# line 1707 "lev_comp.y"


/*lev_comp.y*/
YYCONST yytabelem yyexca[] ={
-1, 1,
	0, -1,
	-2, 0,
-1, 181,
	44, 98,
	-2, 97,
	};
# define YYNPROD 223
# define YYLAST 512
YYCONST yytabelem yyact[]={

   470,   464,   157,   437,   312,   444,   185,   387,    67,   408,
   156,   225,   257,   212,   474,   315,   466,    20,    22,   316,
   313,    47,   104,   105,   103,   107,   152,   158,   183,    21,
   450,   451,   453,   439,    21,   114,   438,   155,   297,   293,
   475,   217,    49,    28,    12,   151,    21,   119,   120,   115,
   154,   153,   226,    60,   425,   440,   399,    21,    21,   318,
    21,   465,    55,    56,   375,   305,   229,   187,   186,   406,
    60,   150,   388,   374,    68,    69,   183,   315,   132,   253,
   440,   316,   313,    61,   135,   356,   466,   384,   409,   381,
   382,   213,   159,   258,   334,   331,   214,   259,   238,    43,
    61,   104,   105,   103,   107,   106,   108,   116,   117,   109,
   321,    74,   322,   407,   114,   118,   110,   121,   122,   317,
   111,   112,   113,   254,   208,   210,   119,   120,   115,   129,
   202,   465,   204,   410,   301,   365,   224,   246,   219,   180,
    64,    66,    65,   239,   422,    44,    36,    34,    17,     8,
     9,    25,   459,   456,   247,   285,   188,   458,   448,   442,
   424,   421,   417,   416,   414,   398,   376,   216,   370,   361,
   359,   347,   344,   324,   307,   302,   298,   218,   294,   287,
   274,   270,   251,   233,   234,   235,   236,   244,   240,   135,
   132,    70,    39,   133,   404,   250,   130,   205,   227,   342,
   134,   339,   337,   352,   131,   149,   268,   255,   256,   264,
   262,   148,   260,   147,   395,    78,   144,   143,   142,   197,
   141,   230,   231,   232,   140,   196,   195,   194,   193,   192,
   189,   178,   177,   176,   175,   174,   173,   172,   171,   170,
   169,   168,   167,   166,   165,   164,   163,   162,   161,   160,
   125,   124,   123,    81,    80,    76,    75,    48,    38,    26,
    18,    15,    14,   182,   292,    54,   296,   354,    98,   471,
   368,   469,   455,   454,   100,    79,    99,   299,   300,    94,
    93,    86,   303,    84,   445,   443,   441,    83,   435,   432,
   429,   221,   423,   319,   222,   420,   415,   412,   411,   403,
   400,   397,   329,   390,   386,   379,   378,   371,   369,   394,
   368,   363,   360,   358,   228,   357,   353,   351,   346,   184,
   179,   343,   341,    77,   340,   336,   328,   325,   311,   221,
   310,   309,   222,   308,   306,   389,   304,   290,   355,   221,
   289,   288,   222,   286,   284,   283,   282,   281,   362,   280,
   385,   279,   278,   271,   367,   366,   377,   335,   332,   345,
   211,   201,   348,   349,   350,   223,   207,   207,   269,   267,
   266,   265,   327,   263,   401,   393,   241,   242,   261,   245,
   199,   198,   191,   190,   228,   249,   127,   248,   126,    50,
    40,    30,   468,   413,   461,   460,   457,   433,   430,   426,
   354,    27,   183,   273,   247,   224,    31,    23,    11,     2,
   220,    16,   277,    10,   402,   372,    13,   276,   431,   275,
   428,   427,   182,   323,   326,    19,   272,    29,   364,   436,
   419,   392,    37,   338,   434,   418,   446,   447,   449,    45,
   452,   391,    51,   102,   101,    97,    96,    95,    92,    91,
    90,    89,    88,    87,    85,    82,   462,   467,   463,   243,
   181,    63,    35,    62,    46,    33,    32,   146,   472,   145,
   139,   473,   138,   137,   476,   136,   383,   330,   333,    59,
    58,   128,    73,    72,    57,    53,    24,    71,    52,    41,
     5,     4,     3,     1,   295,   291,     7,     6,   209,   206,
   203,   200,   396,   320,   237,   252,   405,   380,    42,   314,
   373,   215 };
YYCONST yytabelem yypact[]={

  -112,-10000000,-10000000,  -112,-10000000,-10000000,  -272,  -272,   204,   203,
-10000000,  -115,   202,  -115,  -293,  -293,  -109,   201,  -274,  -109,
   347,-10000000,-10000000,  -118,  -109,   200,   -65,-10000000,   346,-10000000,
  -158,-10000000,  -118,-10000000,-10000000,  -302,   199,-10000000,  -280,   345,
  -274,  -214,-10000000,-10000000,-10000000,-10000000,  -136,-10000000,  -235,-10000000,
   -66,-10000000,  -186,-10000000,-10000000,   198,   197,  -231,-10000000,-10000000,
   196,   195,  -165,-10000000,   194,   193,   192,   344,-10000000,-10000000,
   342,-10000000,-10000000,  -161,-10000000,   -67,   -68,-10000000,  -244,  -244,
  -276,  -276,-10000000,-10000000,-10000000,-10000000,-10000000,-10000000,-10000000,-10000000,
-10000000,-10000000,-10000000,-10000000,-10000000,-10000000,-10000000,-10000000,-10000000,-10000000,
-10000000,-10000000,-10000000,   191,   190,   189,   188,   187,   186,   185,
   184,   183,   182,   181,   180,   179,   178,-10000000,   177,   176,
   175,   174,   173,   -67,   362,   -68,  -243,  -103,-10000000,   172,
-10000000,   339,-10000000,-10000000,   338,-10000000,-10000000,-10000000,-10000000,-10000000,
-10000000,-10000000,-10000000,-10000000,-10000000,-10000000,-10000000,-10000000,-10000000,-10000000,
   171,   170,   169,   168,   167,   161,   337,-10000000,-10000000,   336,
  -173,  -179,  -179,  -207,  -262,    36,   365,    26,    96,    96,
    96,    36,    36,    36,    36,  -160,    36,   365,   365,-10000000,
-10000000,-10000000,-10000000,   -71,-10000000,-10000000,-10000000,-10000000,   335,   364,
   -67,   -68,  -293,   -76,  -180,    36,    36,    36,  -206,  -206,
   334,-10000000,-10000000,-10000000,   119,-10000000,   329,-10000000,-10000000,-10000000,
   118,-10000000,   327,-10000000,-10000000,   326,-10000000,-10000000,   325,-10000000,
-10000000,-10000000,   115,   324,   -77,   309,-10000000,-10000000,   363,   -78,
-10000000,-10000000,-10000000,   308,-10000000,   307,   305,   303,-10000000,-10000000,
   302,-10000000,-10000000,   301,   300,  -104,   299,   -79,-10000000,-10000000,
-10000000,-10000000,   297,-10000000,-10000000,-10000000,-10000000,   296,-10000000,-10000000,
   293,  -264,   -80,  -265,   -82,    36,    36,  -168,   -83,  -206,
   292,  -248,   290,   -84,   289,   287,   286,   284,  -226,  -183,
  -254,    36,  -191,   362,   -85,   283,   114,   282,  -207,    55,
    54,   281,-10000000,-10000000,   109,-10000000,-10000000,-10000000,   108,-10000000,
   280,   278,   106,   277,   -86,-10000000,    96,   274,   -87,    96,
    96,    96,   273,-10000000,-10000000,-10000000,   112,-10000000,-10000000,-10000000,
   272,-10000000,-10000000,-10000000,   359,  -206,-10000000,-10000000,  -217,   271,
   269,   -88,-10000000,   268,   -89,-10000000,    36,-10000000,   267,-10000000,
  -154,  -207,-10000000,  -276,   266,   264,   -90,   226,   263,-10000000,
-10000000,  -239,   -92,  -293,-10000000,   262,   261,  -213,    47,   260,
    32,   259,-10000000,   -12,-10000000,   156,-10000000,   257,   -93,  -257,
   256,  -293,   255,-10000000,-10000000,-10000000,   101,-10000000,  -190,  -170,
   254,-10000000,-10000000,   253,  -235,-10000000,   -94,   252,   -95,-10000000,
   -96,-10000000,   251,-10000000,-10000000,   -97,-10000000,  -131,   248,-10000000,
   -98,-10000000,-10000000,  -259,-10000000,-10000000,-10000000,-10000000,   358,-10000000,
-10000000,  -170,    32,   246,   357,  -293,   245,   356,   244,-10000000,
  -267,-10000000,   242,   -99,   241,-10000000,-10000000,-10000000,   240,  -243,
-10000000,   240,  -100,-10000000,-10000000,  -288,   229,   228,-10000000,-10000000,
-10000000,  -106,   355,  -101,-10000000,  -107,   354,-10000000,   353,-10000000,
-10000000,-10000000,-10000000,  -293,  -242,  -172,-10000000,-10000000,   351,-10000000,
-10000000,-10000000,-10000000,   227,   225,-10000000,-10000000,   225,-10000000,  -172,
-10000000,  -282,-10000000,   225,-10000000,-10000000,-10000000 };
YYCONST yytabelem yypgo[]={

     0,     8,     6,   511,    10,    13,    12,     4,   510,   509,
     5,   508,     9,   507,   506,   505,   504,   503,   408,   401,
   502,    52,   411,   200,   501,   500,   204,   499,   498,     2,
   497,   496,   495,   494,   137,   493,   409,   492,   491,   490,
   407,   406,   489,   488,   487,   486,   485,   196,   193,   265,
   484,   483,   482,   481,   480,   215,   479,   478,     7,   477,
   476,   475,   473,   472,   470,   224,   220,   218,   217,   216,
   469,   467,   213,   211,   205,   466,   465,   464,   463,   462,
   461,   139,   460,   459,   455,   454,   453,   452,   451,   450,
   449,   448,   447,   446,   445,   444,   443,    11,   441,   435,
   434,   197,   433,   431,   430,   429,     3,     1,     0,   428,
   426,   419,   417,   415,   414,   412,   198,   138,   410 };
YYCONST yytabelem yyr1[]={

     0,    35,    35,    36,    36,    37,    37,    38,    39,    31,
    22,    22,    14,    14,    18,    18,    19,    19,    40,    40,
    45,    42,    42,    46,    46,    43,    43,    49,    49,    44,
    44,    51,    52,    52,    53,    53,    34,    50,    50,    56,
    54,    10,    10,    59,    59,    57,    57,    60,    60,    58,
    58,    55,    55,    61,    61,    61,    61,    61,    61,    61,
    61,    61,    61,    61,    61,    61,    62,    63,    64,    15,
    15,    13,    13,    12,    12,    30,    11,    11,    41,    41,
    75,    76,    76,    79,     1,     1,     2,     2,    77,    77,
    80,    80,    80,    47,    47,    48,    48,    81,    83,    81,
    78,    78,    84,    84,    84,    84,    84,    84,    84,    84,
    84,    84,    84,    84,    84,    84,    84,    84,    84,    84,
    84,    84,    98,    65,    99,    99,   100,   100,   100,   100,
   100,    66,    66,   102,   101,   103,   103,   104,   104,   104,
   104,   105,   105,   106,   107,   107,   108,   108,   108,    85,
    67,    67,    86,    92,    93,    94,    74,   110,    88,   111,
    89,   112,   113,    90,   115,    91,   114,   114,    21,    21,
    69,    70,    71,    95,    96,    87,    68,    72,    73,    24,
    24,    24,    27,    27,    27,    32,    32,    33,    33,     3,
     3,   109,     4,     4,    20,    20,    20,    97,    97,    97,
     5,     5,     6,     6,     7,     7,     7,     8,     8,   118,
    28,    25,     9,    82,    23,    26,    29,    16,    16,    17,
    17,   117,   116 };
YYCONST yytabelem yyr2[]={

     0,     0,     2,     2,     4,     2,     2,    11,    15,     7,
     1,    27,     2,     2,     1,     7,     7,     3,     0,     4,
     7,     0,     4,     7,     7,     1,     2,     2,     4,     2,
     2,     3,     0,     4,    11,    11,    15,     5,     5,    25,
    25,     1,     5,    11,     3,    11,     3,    11,     3,    11,
     3,     0,     4,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     7,     7,    19,     2,
     2,     2,     2,     2,     2,    11,     3,     3,     2,     4,
     7,     3,     5,    11,     2,     2,     2,     2,     0,     4,
     7,     7,     7,     3,     7,     3,     7,     3,     1,     8,
     0,     4,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
     2,     2,     1,    19,     0,     4,     5,     5,     5,     5,
     7,     7,     7,     1,    15,     3,     3,     1,    15,    11,
    11,     3,     3,     3,     3,     3,     0,     5,     5,    11,
    11,    15,    15,    11,     3,    11,    11,     1,    17,     1,
    17,     1,     1,    17,     1,    13,     1,     5,     3,    21,
     7,     7,     7,     7,     7,    17,    15,    11,    15,     2,
     3,     2,     2,     3,     2,     2,     3,     2,     3,     3,
     2,     7,     3,     2,     1,     5,     9,     2,     2,     3,
     2,     2,     2,     2,     2,     2,     3,     2,     2,     9,
     9,     9,     9,     2,     3,     3,     2,     2,     2,     2,
     2,    11,    19 };
YYCONST yytabelem yychk[]={

-10000000,   -35,   -36,   -37,   -38,   -39,   -30,   -31,   261,   262,
   -36,   -18,   316,   -18,    58,    58,   -22,   263,    58,   -22,
   -29,   322,   -29,   -40,   -45,   260,    58,   -19,   317,   -40,
    44,   -41,   -75,   -76,   265,   -79,   264,   -40,    58,   257,
    44,   -42,   -11,   257,   303,   -41,   -77,   323,    58,   322,
    44,   -19,   -43,   -46,   -49,   276,   277,   -50,   -54,   -56,
   284,   314,   -78,   -80,   276,   278,   277,    -1,   309,   310,
   257,   -44,   -51,   -52,   297,    58,    58,   -49,   -55,   -55,
    58,    58,   -84,   -65,   -66,   -85,   -67,   -86,   -87,   -88,
   -89,   -90,   -91,   -68,   -69,   -92,   -93,   -94,   -74,   -72,
   -73,   -95,   -96,   268,   266,   267,   270,   269,   271,   274,
   281,   285,   286,   287,   279,   293,   272,   273,   280,   291,
   292,   282,   283,    58,    58,    58,    44,    44,   -53,   290,
   -47,   -26,   257,   -48,   -23,   257,   -61,   -62,   -63,   -64,
   -65,   -66,   -67,   -68,   -69,   -70,   -71,   -72,   -73,   -74,
   315,   289,   270,   295,   294,   281,    -4,   -29,   303,    -4,
    58,    58,    58,    58,    58,    58,    58,    58,    58,    58,
    58,    58,    58,    58,    58,    58,    58,    58,    58,   -47,
   -81,   -82,  -117,    40,   -48,    -2,   311,   310,   259,    58,
    44,    44,    58,    58,    58,    58,    58,    58,    44,    44,
   -24,   -23,   303,   -25,   305,  -101,   -27,   -26,   303,   -28,
   304,  -101,    -5,   298,   303,    -3,   -29,   303,   -97,  -117,
  -118,   303,   306,  -116,    40,   -97,   -21,  -116,   288,    40,
   -21,   -21,   -21,   -97,   -97,   -97,   -97,   -16,   258,   303,
   -97,  -116,  -116,   -83,   258,    44,   -34,    40,   -47,   -48,
   -29,   258,   -15,   259,   303,   -97,   -97,    -6,   299,   303,
    -6,    44,    91,    44,    91,    44,    44,    44,    91,    44,
   258,    44,  -110,    40,   258,  -111,  -112,  -115,    44,    44,
    44,    44,    44,    44,    44,   259,    44,   258,    44,    44,
    44,   -32,   -29,   303,   258,   -33,   -29,   303,   258,   -97,
   -97,   302,   258,    -6,    44,   313,    44,   258,    44,    44,
    44,    44,    -7,   308,    -9,   303,   307,   302,   313,   -97,
   -17,   301,   303,   -81,   258,    44,   -34,   258,    44,    -5,
   -59,    40,   303,   -57,    40,   303,    44,    93,  -102,    93,
    44,    44,    93,    44,   258,   -21,    44,   258,   -21,   -21,
   -21,    44,    91,    44,    41,    -6,   302,    44,    44,   258,
    44,   258,   -97,    44,  -109,   289,    -5,    -4,    44,    44,
   258,    44,  -113,    -8,   312,   303,   258,   -29,    44,    44,
   -13,   302,   303,   -60,    40,   303,    44,   -58,    40,   303,
    44,   -98,  -103,   -97,   321,    58,   -20,    44,   258,   313,
    44,   -29,  -114,    44,    93,   -14,   259,   303,   -12,   258,
   303,    44,    44,    -1,   258,    44,   258,   258,   -99,  -104,
    44,   258,   275,    44,   258,   313,    41,   -12,   -58,    44,
    41,   -29,    44,    41,  -100,    44,  -105,  -106,   303,   300,
   322,    44,   258,    44,   -10,    44,    -2,   -10,   258,   -29,
   318,   319,    -7,   320,    44,    44,   259,    41,   258,   259,
    41,    41,   -29,  -106,  -107,   303,   258,  -107,    41,    44,
  -108,    44,  -108,  -107,   296,   322,  -108 };
YYCONST yytabelem yydef[]={

     1,    -2,     2,     3,     5,     6,    14,    14,     0,     0,
     4,    10,     0,    10,     0,     0,    18,     0,     0,    18,
     0,   216,     9,     0,    18,     0,     0,    15,    17,    21,
     0,     7,    78,    88,    81,     0,     0,    19,     0,     0,
     0,    25,    75,    76,    77,    79,   100,    82,     0,    20,
     0,    16,    32,    22,    26,     0,     0,    27,    51,    51,
     0,     0,    80,    89,     0,     0,     0,     0,    84,    85,
     0,     8,    29,    30,    31,     0,     0,    28,    37,    38,
     0,     0,   101,   102,   103,   104,   105,   106,   107,   108,
   109,   110,   111,   112,   113,   114,   115,   116,   117,   118,
   119,   120,   121,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,   154,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,    33,     0,
    23,    93,   215,    24,    95,   214,    52,    53,    54,    55,
    56,    57,    58,    59,    60,    61,    62,    63,    64,    65,
     0,     0,     0,     0,     0,     0,     0,   192,   193,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,    90,
    91,    -2,   213,     0,    92,    83,    86,    87,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,   179,   180,   181,     0,   131,     0,   182,   183,   184,
     0,   132,     0,   200,   201,     0,   189,   190,     0,   197,
   198,   199,     0,     0,     0,     0,   157,   168,     0,     0,
   159,   161,   164,     0,   170,     0,     0,     0,   217,   218,
     0,   173,   174,     0,     0,     0,     0,     0,    94,    96,
    66,    67,     0,    69,    70,   171,   172,     0,   202,   203,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,     0,     0,     0,     0,     0,     0,     0,     0,
     0,     0,   185,   186,     0,   133,   187,   188,     0,   149,
   150,     0,     0,     0,     0,   156,     0,     0,     0,     0,
     0,     0,     0,   204,   205,   206,     0,   153,   155,   177,
     0,   219,   220,    99,     0,     0,    34,    35,     0,     0,
     0,     0,    44,     0,     0,    46,     0,   211,     0,   210,
     0,     0,   209,     0,     0,     0,     0,     0,     0,   162,
   165,     0,     0,     0,   221,     0,     0,     0,     0,     0,
     0,     0,   122,     0,   151,     0,   152,   194,     0,     0,
     0,     0,   166,   176,   207,   208,     0,   178,     0,     0,
     0,    71,    72,     0,     0,    48,     0,     0,     0,    50,
     0,   124,   137,   135,   136,     0,   175,     0,     0,   158,
     0,   160,   163,     0,   212,    11,    12,    13,     0,    73,
    74,     0,     0,     0,     0,     0,     0,     0,   123,   134,
     0,   191,   195,     0,     0,   167,    36,    68,    41,     0,
    43,    41,     0,    45,   125,     0,     0,     0,   141,   142,
   143,     0,     0,     0,    40,     0,     0,    39,     0,   126,
   127,   128,   129,     0,     0,     0,   196,   222,     0,    42,
    47,    49,   130,     0,   146,   144,   145,   146,   169,     0,
   139,     0,   140,   146,   147,   148,   138 };
typedef struct
#ifdef __cplusplus
	yytoktype
#endif
{ char *t_name; int t_val; } yytoktype;
#ifndef YYDEBUG
#	define YYDEBUG	0	/* don't allow debugging */
#endif

#if YYDEBUG

YYCONST yytoktype yytoks[] =
{
	"CHAR",	257,
	"INTEGER",	258,
	"BOOLEAN",	259,
	"MESSAGE_ID",	260,
	"MAZE_ID",	261,
	"LEVEL_ID",	262,
	"LEV_INIT_ID",	263,
	"GEOMETRY_ID",	264,
	"NOMAP_ID",	265,
	"OBJECT_ID",	266,
	"COBJECT_ID",	267,
	"MONSTER_ID",	268,
	"TRAP_ID",	269,
	"DOOR_ID",	270,
	"DRAWBRIDGE_ID",	271,
	"MAZEWALK_ID",	272,
	"WALLIFY_ID",	273,
	"REGION_ID",	274,
	"FILLING",	275,
	"RANDOM_OBJECTS_ID",	276,
	"RANDOM_MONSTERS_ID",	277,
	"RANDOM_PLACES_ID",	278,
	"ALTAR_ID",	279,
	"LADDER_ID",	280,
	"STAIR_ID",	281,
	"NON_DIGGABLE_ID",	282,
	"NON_PASSWALL_ID",	283,
	"ROOM_ID",	284,
	"PORTAL_ID",	285,
	"TELEPRT_ID",	286,
	"BRANCH_ID",	287,
	"LEV",	288,
	"CHANCE_ID",	289,
	"CORRIDOR_ID",	290,
	"GOLD_ID",	291,
	"ENGRAVING_ID",	292,
	"FOUNTAIN_ID",	293,
	"POOL_ID",	294,
	"SINK_ID",	295,
	"NONE",	296,
	"RAND_CORRIDOR_ID",	297,
	"DOOR_STATE",	298,
	"LIGHT_STATE",	299,
	"CURSE_TYPE",	300,
	"ENGRAVING_TYPE",	301,
	"DIRECTION",	302,
	"RANDOM_TYPE",	303,
	"O_REGISTER",	304,
	"M_REGISTER",	305,
	"P_REGISTER",	306,
	"A_REGISTER",	307,
	"ALIGNMENT",	308,
	"LEFT_OR_RIGHT",	309,
	"CENTER",	310,
	"TOP_OR_BOT",	311,
	"ALTAR_TYPE",	312,
	"UP_OR_DOWN",	313,
	"SUBROOM_ID",	314,
	"NAME_ID",	315,
	"FLAGS_ID",	316,
	"FLAG_TYPE",	317,
	"MON_ATTITUDE",	318,
	"MON_ALERTNESS",	319,
	"MON_APPEARANCE",	320,
	"CONTAINED",	321,
	",",	44,
	":",	58,
	"(",	40,
	")",	41,
	"[",	91,
	"]",	93,
	"STRING",	322,
	"MAP_ID",	323,
	"-unknown-",	-1	/* ends search */
};

YYCONST char * yyreds[] =
{
	"-no such reduction-",
	"file : /* empty */",
	"file : levels",
	"levels : level",
	"levels : level levels",
	"level : maze_level",
	"level : room_level",
	"maze_level : maze_def flags lev_init messages regions",
	"room_level : level_def flags lev_init messages rreg_init rooms corridors_def",
	"level_def : LEVEL_ID ':' string",
	"lev_init : /* empty */",
	"lev_init : LEV_INIT_ID ':' CHAR ',' CHAR ',' BOOLEAN ',' BOOLEAN ',' light_state ',' walled",
	"walled : BOOLEAN",
	"walled : RANDOM_TYPE",
	"flags : /* empty */",
	"flags : FLAGS_ID ':' flag_list",
	"flag_list : FLAG_TYPE ',' flag_list",
	"flag_list : FLAG_TYPE",
	"messages : /* empty */",
	"messages : message messages",
	"message : MESSAGE_ID ':' STRING",
	"rreg_init : /* empty */",
	"rreg_init : rreg_init init_rreg",
	"init_rreg : RANDOM_OBJECTS_ID ':' object_list",
	"init_rreg : RANDOM_MONSTERS_ID ':' monster_list",
	"rooms : /* empty */",
	"rooms : roomlist",
	"roomlist : aroom",
	"roomlist : aroom roomlist",
	"corridors_def : random_corridors",
	"corridors_def : corridors",
	"random_corridors : RAND_CORRIDOR_ID",
	"corridors : /* empty */",
	"corridors : corridors corridor",
	"corridor : CORRIDOR_ID ':' corr_spec ',' corr_spec",
	"corridor : CORRIDOR_ID ':' corr_spec ',' INTEGER",
	"corr_spec : '(' INTEGER ',' DIRECTION ',' door_pos ')'",
	"aroom : room_def room_details",
	"aroom : subroom_def room_details",
	"subroom_def : SUBROOM_ID ':' room_type ',' light_state ',' subroom_pos ',' room_size ',' string roomfill",
	"room_def : ROOM_ID ':' room_type ',' light_state ',' room_pos ',' room_align ',' room_size roomfill",
	"roomfill : /* empty */",
	"roomfill : ',' BOOLEAN",
	"room_pos : '(' INTEGER ',' INTEGER ')'",
	"room_pos : RANDOM_TYPE",
	"subroom_pos : '(' INTEGER ',' INTEGER ')'",
	"subroom_pos : RANDOM_TYPE",
	"room_align : '(' h_justif ',' v_justif ')'",
	"room_align : RANDOM_TYPE",
	"room_size : '(' INTEGER ',' INTEGER ')'",
	"room_size : RANDOM_TYPE",
	"room_details : /* empty */",
	"room_details : room_details room_detail",
	"room_detail : room_name",
	"room_detail : room_chance",
	"room_detail : room_door",
	"room_detail : monster_detail",
	"room_detail : object_detail",
	"room_detail : trap_detail",
	"room_detail : altar_detail",
	"room_detail : fountain_detail",
	"room_detail : sink_detail",
	"room_detail : pool_detail",
	"room_detail : gold_detail",
	"room_detail : engraving_detail",
	"room_detail : stair_detail",
	"room_name : NAME_ID ':' string",
	"room_chance : CHANCE_ID ':' INTEGER",
	"room_door : DOOR_ID ':' secret ',' door_state ',' door_wall ',' door_pos",
	"secret : BOOLEAN",
	"secret : RANDOM_TYPE",
	"door_wall : DIRECTION",
	"door_wall : RANDOM_TYPE",
	"door_pos : INTEGER",
	"door_pos : RANDOM_TYPE",
	"maze_def : MAZE_ID ':' string ',' filling",
	"filling : CHAR",
	"filling : RANDOM_TYPE",
	"regions : aregion",
	"regions : aregion regions",
	"aregion : map_definition reg_init map_details",
	"map_definition : NOMAP_ID",
	"map_definition : map_geometry MAP_ID",
	"map_geometry : GEOMETRY_ID ':' h_justif ',' v_justif",
	"h_justif : LEFT_OR_RIGHT",
	"h_justif : CENTER",
	"v_justif : TOP_OR_BOT",
	"v_justif : CENTER",
	"reg_init : /* empty */",
	"reg_init : reg_init init_reg",
	"init_reg : RANDOM_OBJECTS_ID ':' object_list",
	"init_reg : RANDOM_PLACES_ID ':' place_list",
	"init_reg : RANDOM_MONSTERS_ID ':' monster_list",
	"object_list : object",
	"object_list : object ',' object_list",
	"monster_list : monster",
	"monster_list : monster ',' monster_list",
	"place_list : place",
	"place_list : place",
	"place_list : place ',' place_list",
	"map_details : /* empty */",
	"map_details : map_details map_detail",
	"map_detail : monster_detail",
	"map_detail : object_detail",
	"map_detail : door_detail",
	"map_detail : trap_detail",
	"map_detail : drawbridge_detail",
	"map_detail : region_detail",
	"map_detail : stair_region",
	"map_detail : portal_region",
	"map_detail : teleprt_region",
	"map_detail : branch_region",
	"map_detail : altar_detail",
	"map_detail : fountain_detail",
	"map_detail : mazewalk_detail",
	"map_detail : wallify_detail",
	"map_detail : ladder_detail",
	"map_detail : stair_detail",
	"map_detail : gold_detail",
	"map_detail : engraving_detail",
	"map_detail : diggable_detail",
	"map_detail : passwall_detail",
	"monster_detail : MONSTER_ID ':' monster_c ',' m_name ',' coordinate",
	"monster_detail : MONSTER_ID ':' monster_c ',' m_name ',' coordinate monster_infos",
	"monster_infos : /* empty */",
	"monster_infos : monster_infos monster_info",
	"monster_info : ',' string",
	"monster_info : ',' MON_ATTITUDE",
	"monster_info : ',' MON_ALERTNESS",
	"monster_info : ',' alignment",
	"monster_info : ',' MON_APPEARANCE string",
	"object_detail : OBJECT_ID ':' object_desc",
	"object_detail : COBJECT_ID ':' object_desc",
	"object_desc : object_c ',' o_name",
	"object_desc : object_c ',' o_name ',' object_where object_infos",
	"object_where : coordinate",
	"object_where : CONTAINED",
	"object_infos : /* empty */",
	"object_infos : ',' curse_state ',' monster_id ',' enchantment optional_name",
	"object_infos : ',' curse_state ',' enchantment optional_name",
	"object_infos : ',' monster_id ',' enchantment optional_name",
	"curse_state : RANDOM_TYPE",
	"curse_state : CURSE_TYPE",
	"monster_id : STRING",
	"enchantment : RANDOM_TYPE",
	"enchantment : INTEGER",
	"optional_name : /* empty */",
	"optional_name : ',' NONE",
	"optional_name : ',' STRING",
	"door_detail : DOOR_ID ':' door_state ',' coordinate",
	"trap_detail : TRAP_ID ':' trap_name ',' coordinate",
	"trap_detail : TRAP_ID ':' trap_name ',' coordinate ',' trap_chance",
	"drawbridge_detail : DRAWBRIDGE_ID ':' coordinate ',' DIRECTION ',' door_state",
	"mazewalk_detail : MAZEWALK_ID ':' coordinate ',' DIRECTION",
	"wallify_detail : WALLIFY_ID",
	"ladder_detail : LADDER_ID ':' coordinate ',' UP_OR_DOWN",
	"stair_detail : STAIR_ID ':' coordinate ',' UP_OR_DOWN",
	"stair_region : STAIR_ID ':' lev_region",
	"stair_region : STAIR_ID ':' lev_region ',' lev_region ',' UP_OR_DOWN",
	"portal_region : PORTAL_ID ':' lev_region",
	"portal_region : PORTAL_ID ':' lev_region ',' lev_region ',' string",
	"teleprt_region : TELEPRT_ID ':' lev_region",
	"teleprt_region : TELEPRT_ID ':' lev_region ',' lev_region",
	"teleprt_region : TELEPRT_ID ':' lev_region ',' lev_region teleprt_detail",
	"branch_region : BRANCH_ID ':' lev_region",
	"branch_region : BRANCH_ID ':' lev_region ',' lev_region",
	"teleprt_detail : /* empty */",
	"teleprt_detail : ',' UP_OR_DOWN",
	"lev_region : region",
	"lev_region : LEV '(' INTEGER ',' INTEGER ',' INTEGER ',' INTEGER ')'",
	"fountain_detail : FOUNTAIN_ID ':' coordinate",
	"sink_detail : SINK_ID ':' coordinate",
	"pool_detail : POOL_ID ':' coordinate",
	"diggable_detail : NON_DIGGABLE_ID ':' region",
	"passwall_detail : NON_PASSWALL_ID ':' region",
	"region_detail : REGION_ID ':' region ',' light_state ',' room_type prefilled",
	"altar_detail : ALTAR_ID ':' coordinate ',' alignment ',' altar_type",
	"gold_detail : GOLD_ID ':' amount ',' coordinate",
	"engraving_detail : ENGRAVING_ID ':' coordinate ',' engraving_type ',' string",
	"monster_c : monster",
	"monster_c : RANDOM_TYPE",
	"monster_c : m_register",
	"object_c : object",
	"object_c : RANDOM_TYPE",
	"object_c : o_register",
	"m_name : string",
	"m_name : RANDOM_TYPE",
	"o_name : string",
	"o_name : RANDOM_TYPE",
	"trap_name : string",
	"trap_name : RANDOM_TYPE",
	"trap_chance : CHANCE_ID ':' INTEGER",
	"room_type : string",
	"room_type : RANDOM_TYPE",
	"prefilled : /* empty */",
	"prefilled : ',' FILLING",
	"prefilled : ',' FILLING ',' BOOLEAN",
	"coordinate : coord",
	"coordinate : p_register",
	"coordinate : RANDOM_TYPE",
	"door_state : DOOR_STATE",
	"door_state : RANDOM_TYPE",
	"light_state : LIGHT_STATE",
	"light_state : RANDOM_TYPE",
	"alignment : ALIGNMENT",
	"alignment : a_register",
	"alignment : RANDOM_TYPE",
	"altar_type : ALTAR_TYPE",
	"altar_type : RANDOM_TYPE",
	"p_register : P_REGISTER '[' INTEGER ']'",
	"o_register : O_REGISTER '[' INTEGER ']'",
	"m_register : M_REGISTER '[' INTEGER ']'",
	"a_register : A_REGISTER '[' INTEGER ']'",
	"place : coord",
	"monster : CHAR",
	"object : CHAR",
	"string : STRING",
	"amount : INTEGER",
	"amount : RANDOM_TYPE",
	"engraving_type : ENGRAVING_TYPE",
	"engraving_type : RANDOM_TYPE",
	"coord : '(' INTEGER ',' INTEGER ')'",
	"region : '(' INTEGER ',' INTEGER ',' INTEGER ',' INTEGER ')'",
};
#endif /* YYDEBUG */
#ident	"@(#) yaccpar 20.1 94/12/04 "
#ident	"@(#)yacc:common/yaccpar	1.26"

/*
** Skeleton parser driver for yacc output
*/

#include <unistd.h> 	/* to get a definition for gettxt so C++ doesn't
				complain */

/*
** yacc user known macros and defines
*/
#define YYERROR		goto yyerrlab
#define YYACCEPT	return(0)
#define YYABORT		return(1)
#define YYBACKUP( newtoken, newvalue )\
{\
	if ( yychar >= 0 || ( yyr2[ yytmp ] >> 1 ) != 1 )\
	{\
		yyerror( gettxt("uxlibc:78", "Syntax error - cannot backup") );\
		goto yyerrlab;\
	}\
	yychar = newtoken;\
	yystate = *yyps;\
	yylval = newvalue;\
	goto yynewstate;\
}
#define YYRECOVERING()	(!!yyerrflag)
#define YYNEW(type)	(char *) malloc(sizeof(type) * yynewmax)
#define YYCOPY(to, from, type) \
	(type *) memcpy(to, (char *) from, yynewmax * sizeof(type))
#define YYENLARGE( from, type) \
	(type *) realloc((char *) from, yynewmax * sizeof(type))
#ifndef YYDEBUG
#	define YYDEBUG	1	/* make debugging available */
#endif

/*
** user known globals
*/
int yydebug;			/* set to 1 to get debugging */

/*
** driver internal defines
*/
#define YYFLAG		(-10000000)

/*
** global variables used by the parser
*/
YYSTYPE *yypv;			/* top of value stack */
int *yyps;			/* top of state stack */

int yystate;			/* current state */
int yytmp;			/* extra var (lasts between blocks) */

int yynerrs;			/* number of errors */
int yyerrflag;			/* error recovery flag */
int yychar;			/* current input token number */

#ifndef __HAVE_GETTXT__	/* Message catalogue stuff */
#define gettxt(a, b) b
#endif

#if defined(__cplusplus) && ! defined(YACC_HAS_CPLUS_LINKAGE)
extern "C"
#endif

/*
** yyparse - return 0 if worked, 1 if syntax error not recovered from
*/

#if defined(__STDC__) || defined(__cplusplus)
int yyparse(void)
#else
int yyparse()
#endif
{
	register YYSTYPE *yypvt;	/* top of value stack for $vars */
	int *yys_end;

	/*
	** Initialize externals - yyparse may be called more than once
	*/
	yypv = &yyv[-1];
	yyps = &yys[-1];
	yystate = 0;
	yytmp = 0;
	yynerrs = 0;
	yyerrflag = 0;
	yychar = -1;

#if YYMAXDEPTH <= 0
	if (yymaxdepth <= 0)
	{
		if ((yymaxdepth = YYEXPAND(0)) <= 0)
		{
			yyerror(gettxt("uxlibc:79", "Yacc initialization error"));
			YYABORT;
		}
	}
#endif
	yys_end = &yys[yymaxdepth];

	goto yystack;
	{
		register YYSTYPE *yy_pv;	/* top of value stack */
		register int *yy_ps;		/* top of state stack */
		register int yy_state;		/* current state */
		register int  yy_n;		/* internal state number info */

		/*
		** get globals into registers.
		** branch to here only if YYBACKUP was called.
		*/
	yynewstate:
		yy_pv = yypv;
		yy_ps = yyps;
		yy_state = yystate;
		goto yy_newstate;

		/*
		** get globals into registers.
		** either we just started, or we just finished a reduction
		*/
	yystack:
		yy_pv = yypv;
		yy_ps = yyps;
		yy_state = yystate;

		/*
		** top of for (;;) loop while no reductions done
		*/
	yy_stack:
		/*
		** put a state and value onto the stacks
		*/
#if YYDEBUG
		/*
		** if debugging, look up token value in list of value vs.
		** name pairs.  0 and negative (-1) are special values.
		** Note: linear search is used since time is not a real
		** consideration while debugging.
		*/
		if ( yydebug )
		{
			register int yy_i;

			printf( "State %d, token ", yy_state );
			if ( yychar == 0 )
				printf( "end-of-file\n" );
			else if ( yychar < 0 )
				printf( "-none-\n" );
			else
			{
				for ( yy_i = 0; yytoks[yy_i].t_val >= 0;
					yy_i++ )
				{
					if ( yytoks[yy_i].t_val == yychar )
						break;
				}
				printf( "%s\n", yytoks[yy_i].t_name );
			}
		}
#endif /* YYDEBUG */
		if ( ++yy_ps >= yys_end )	/* room on stack? */
		{
			int yynewmax;
			long byte_offset;
			int *yys_base = yys;
			YYSTYPE *yyv_base = yyv;
#ifdef YYEXPAND
			yynewmax = YYEXPAND(yymaxdepth);
#else
			yynewmax = 2 * yymaxdepth;	/* double table size */
			if (yymaxdepth == YYMAXDEPTH)	/* first time growth */
			{
				char *newyys = YYNEW(int);
				char *newyyv = YYNEW(YYSTYPE);
				if (newyys != 0 && newyyv != 0)
				{
					yys = YYCOPY(newyys, yys, int);
					yyv = YYCOPY(newyyv, yyv, YYSTYPE);
				}
				else
					yynewmax = 0;	/* failed */
			}
			else				/* not first time */
			{
				yys = YYENLARGE(yys, int);
				yyv = YYENLARGE(yyv, YYSTYPE);
				if (yys == 0 || yyv == 0)
					yynewmax = 0;	/* failed */
			}
#endif
			if (yynewmax <= yymaxdepth)	/* tables not expanded */
			{
				yyerror( gettxt("uxlibc:80", "Yacc stack overflow") );
				YYABORT;
			}
			yymaxdepth = yynewmax;
			yys_end = &yys[yymaxdepth];

			/* reset pointers into yys */
			byte_offset = (char *)yys - (char *)yys_base;
			yy_ps = (int *)(byte_offset + (char *)yy_ps);
			yyps = (int *)(byte_offset + (char *)yyps);

			/* reset pointers into yyv */
			byte_offset = (char *)yyv - (char *)yyv_base;
			yypvt = (YYSTYPE *)(byte_offset + (char *)yypvt);
			yy_pv = (YYSTYPE *)(byte_offset + (char *)yy_pv);
			yypv = (YYSTYPE *)(byte_offset + (char *)yypv);
		}
		*yy_ps = yy_state;
		*++yy_pv = yyval;

		/*
		** we have a new state - find out what to do
		*/
	yy_newstate:
		if ( ( yy_n = yypact[ yy_state ] ) <= YYFLAG )
			goto yydefault;		/* simple state */
#if YYDEBUG
		/*
		** if debugging, need to mark whether new token grabbed
		*/
		yytmp = yychar < 0;
#endif
		if ( ( yychar < 0 ) && ( ( yychar = yylex() ) < 0 ) )
			yychar = 0;		/* reached EOF */
#if YYDEBUG
		if ( yydebug && yytmp )
		{
			register int yy_i;

			printf( "Received token " );
			if ( yychar == 0 )
				printf( "end-of-file\n" );
			else if ( yychar < 0 )
				printf( "-none-\n" );
			else
			{
				for ( yy_i = 0; yytoks[yy_i].t_val >= 0;
					yy_i++ )
				{
					if ( yytoks[yy_i].t_val == yychar )
						break;
				}
				printf( "%s\n", yytoks[yy_i].t_name );
			}
		}
#endif /* YYDEBUG */
		if ( ( ( yy_n += yychar ) < 0 ) || ( yy_n >= YYLAST ) )
			goto yydefault;
		if ( yychk[ yy_n = yyact[ yy_n ] ] == yychar )	/*valid shift*/
		{
			yychar = -1;
			yyval = yylval;
			yy_state = yy_n;
			if ( yyerrflag > 0 )
				yyerrflag--;
			goto yy_stack;
		}

	yydefault:
		if ( ( yy_n = yydef[ yy_state ] ) == -2 )
		{
#if YYDEBUG
			yytmp = yychar < 0;
#endif
			if ( ( yychar < 0 ) && ( ( yychar = yylex() ) < 0 ) )
				yychar = 0;		/* reached EOF */
#if YYDEBUG
			if ( yydebug && yytmp )
			{
				register int yy_i;

				printf( "Received token " );
				if ( yychar == 0 )
					printf( "end-of-file\n" );
				else if ( yychar < 0 )
					printf( "-none-\n" );
				else
				{
					for ( yy_i = 0;
						yytoks[yy_i].t_val >= 0;
						yy_i++ )
					{
						if ( yytoks[yy_i].t_val
							== yychar )
						{
							break;
						}
					}
					printf( "%s\n", yytoks[yy_i].t_name );
				}
			}
#endif /* YYDEBUG */
			/*
			** look through exception table
			*/
			{
				register yytabelem *yyxi = (yytabelem *)yyexca;

				while ( ( *yyxi != -1 ) ||
					( yyxi[1] != yy_state ) )
				{
					yyxi += 2;
				}
				while ( ( *(yyxi += 2) >= 0 ) &&
					( *yyxi != yychar ) )
					;
				if ( ( yy_n = yyxi[1] ) < 0 )
					YYACCEPT;
			}
		}

		/*
		** check for syntax error
		*/
		if ( yy_n == 0 )	/* have an error */
		{
			/* no worry about speed here! */
			switch ( yyerrflag )
			{
			case 0:		/* new error */
				yyerror( gettxt("uxlibc:81", "Syntax error") );
				goto skip_init;
			yyerrlab:
				/*
				** get globals into registers.
				** we have a user generated syntax type error
				*/
				yy_pv = yypv;
				yy_ps = yyps;
				yy_state = yystate;
				yynerrs++;
				/* FALLTHRU */
			skip_init:
			case 1:
			case 2:		/* incompletely recovered error */
					/* try again... */
				yyerrflag = 3;
				/*
				** find state where "error" is a legal
				** shift action
				*/
				while ( yy_ps >= yys )
				{
					yy_n = yypact[ *yy_ps ] + YYERRCODE;
					if ( yy_n >= 0 && yy_n < YYLAST &&
						yychk[yyact[yy_n]] == YYERRCODE)					{
						/*
						** simulate shift of "error"
						*/
						yy_state = yyact[ yy_n ];
						goto yy_stack;
					}
					/*
					** current state has no shift on
					** "error", pop stack
					*/
#if YYDEBUG
#	define _POP_ "Error recovery pops state %d, uncovers state %d\n"
					if ( yydebug )
						printf( _POP_, *yy_ps,
							yy_ps[-1] );
#	undef _POP_
#endif
					yy_ps--;
					yy_pv--;
				}
				/*
				** there is no state on stack with "error" as
				** a valid shift.  give up.
				*/
				YYABORT;
			case 3:		/* no shift yet; eat a token */
#if YYDEBUG
				/*
				** if debugging, look up token in list of
				** pairs.  0 and negative shouldn't occur,
				** but since timing doesn't matter when
				** debugging, it doesn't hurt to leave the
				** tests here.
				*/
				if ( yydebug )
				{
					register int yy_i;

					printf( "Error recovery discards " );
					if ( yychar == 0 )
						printf( "token end-of-file\n" );
					else if ( yychar < 0 )
						printf( "token -none-\n" );
					else
					{
						for ( yy_i = 0;
							yytoks[yy_i].t_val >= 0;
							yy_i++ )
						{
							if ( yytoks[yy_i].t_val
								== yychar )
							{
								break;
							}
						}
						printf( "token %s\n",
							yytoks[yy_i].t_name );
					}
				}
#endif /* YYDEBUG */
				if ( yychar == 0 )	/* reached EOF. quit */
					YYABORT;
				yychar = -1;
				goto yy_newstate;
			}
		}/* end if ( yy_n == 0 ) */
		/*
		** reduction by production yy_n
		** put stack tops, etc. so things right after switch
		*/
#if YYDEBUG
		/*
		** if debugging, print the string that is the user's
		** specification of the reduction which is just about
		** to be done.
		*/
		if ( yydebug )
			printf( "Reduce by (%d) \"%s\"\n",
				yy_n, yyreds[ yy_n ] );
#endif
		yytmp = yy_n;			/* value to switch over */
		yypvt = yy_pv;			/* $vars top of value stack */
		/*
		** Look in goto table for next state
		** Sorry about using yy_state here as temporary
		** register variable, but why not, if it works...
		** If yyr2[ yy_n ] doesn't have the low order bit
		** set, then there is no action to be done for
		** this reduction.  So, no saving & unsaving of
		** registers done.  The only difference between the
		** code just after the if and the body of the if is
		** the goto yy_stack in the body.  This way the test
		** can be made before the choice of what to do is needed.
		*/
		{
			/* length of production doubled with extra bit */
			register int yy_len = yyr2[ yy_n ];

			if ( !( yy_len & 01 ) )
			{
				yy_len >>= 1;
				yyval = ( yy_pv -= yy_len )[1];	/* $$ = $1 */
				yy_state = yypgo[ yy_n = yyr1[ yy_n ] ] +
					*( yy_ps -= yy_len ) + 1;
				if ( yy_state >= YYLAST ||
					yychk[ yy_state =
					yyact[ yy_state ] ] != -yy_n )
				{
					yy_state = yyact[ yypgo[ yy_n ] ];
				}
				goto yy_stack;
			}
			yy_len >>= 1;
			yyval = ( yy_pv -= yy_len )[1];	/* $$ = $1 */
			yy_state = yypgo[ yy_n = yyr1[ yy_n ] ] +
				*( yy_ps -= yy_len ) + 1;
			if ( yy_state >= YYLAST ||
				yychk[ yy_state = yyact[ yy_state ] ] != -yy_n )
			{
				yy_state = yyact[ yypgo[ yy_n ] ];
			}
		}
					/* save until reenter driver code */
		yystate = yy_state;
		yyps = yy_ps;
		yypv = yy_pv;
	}
	/*
	** code supplied by user is placed in this switch
	*/
	switch( yytmp )
	{
		
case 7:
# line 188 "lev_comp.y"
{
			unsigned i;

			if (fatal_error > 0) {
				(void) fprintf(stderr,
				"%s : %d errors detected. No output created!\n",
					fname, fatal_error);
			} else {
				maze.flags = yypvt[-3].i;
				(void) memcpy((genericptr_t)&(maze.init_lev),
						(genericptr_t)&(init_lev),
						sizeof(lev_init));
				maze.numpart = npart;
				maze.parts = NewTab(mazepart, npart);
				for(i=0;i<npart;i++)
				    maze.parts[i] = tmppart[i];
				if (!write_level_file(yypvt[-4].map, (splev *)0, &maze)) {
					yyerror("Can't open output file!!");
					exit(EXIT_FAILURE);
				}
				npart = 0;
			}
			Free(yypvt[-4].map);
		  } break;
case 8:
# line 215 "lev_comp.y"
{
			unsigned i;

			if (fatal_error > 0) {
			    (void) fprintf(stderr,
			      "%s : %d errors detected. No output created!\n",
					fname, fatal_error);
			} else {
				special_lev.flags = (long) yypvt[-5].i;
				(void) memcpy(
					(genericptr_t)&(special_lev.init_lev),
					(genericptr_t)&(init_lev),
					sizeof(lev_init));
				special_lev.nroom = nrooms;
				special_lev.rooms = NewTab(room, nrooms);
				for(i=0; i<nrooms; i++)
				    special_lev.rooms[i] = tmproom[i];
				special_lev.ncorr = ncorridor;
				special_lev.corrs = NewTab(corridor, ncorridor);
				for(i=0; i<ncorridor; i++)
				    special_lev.corrs[i] = tmpcor[i];
				if (check_subrooms()) {
				    if (!write_level_file(yypvt[-6].map, &special_lev,
							  (specialmaze *)0)) {
					yyerror("Can't open output file!!");
					exit(EXIT_FAILURE);
				    }
				}
				free_rooms(&special_lev);
				nrooms = 0;
				ncorridor = 0;
			}
			Free(yypvt[-6].map);
		  } break;
case 9:
# line 252 "lev_comp.y"
{
			if (index(yypvt[-0].map, '.'))
			    yyerror("Invalid dot ('.') in level name.");
			if ((int) strlen(yypvt[-0].map) > 8)
			    yyerror("Level names limited to 8 characters.");
			yyval.map = yypvt[-0].map;
			special_lev.nrmonst = special_lev.nrobjects = 0;
			n_mlist = n_olist = 0;
		  } break;
case 10:
# line 264 "lev_comp.y"
{
			/* in case we're processing multiple files,
			   explicitly clear any stale settings */
			(void) memset((genericptr_t) &init_lev, 0,
					sizeof init_lev);
			init_lev.init_present = FALSE;
			yyval.i = 0;
		  } break;
case 11:
# line 273 "lev_comp.y"
{
			init_lev.init_present = TRUE;
			init_lev.fg = what_map_char((char) yypvt[-10].i);
			if (init_lev.fg == INVALID_TYPE)
			    yyerror("Invalid foreground type.");
			init_lev.bg = what_map_char((char) yypvt[-8].i);
			if (init_lev.bg == INVALID_TYPE)
			    yyerror("Invalid background type.");
			init_lev.smoothed = yypvt[-6].i;
			init_lev.joined = yypvt[-4].i;
			init_lev.lit = yypvt[-2].i;
			init_lev.walled = yypvt[-0].i;
			yyval.i = 1;
		  } break;
case 14:
# line 294 "lev_comp.y"
{
			yyval.i = 0;
		  } break;
case 15:
# line 298 "lev_comp.y"
{
			yyval.i = lev_flags;
			lev_flags = 0;	/* clear for next user */
		  } break;
case 16:
# line 305 "lev_comp.y"
{
			lev_flags |= yypvt[-2].i;
		  } break;
case 17:
# line 309 "lev_comp.y"
{
			lev_flags |= yypvt[-0].i;
		  } break;
case 20:
# line 319 "lev_comp.y"
{
			int i, j;

			i = (int) strlen(yypvt[-0].map) + 1;
			j = (int) strlen(tmpmessage);
			if (i + j > 255) {
			   yyerror("Message string too long (>256 characters)");
			} else {
			    if (j) tmpmessage[j++] = '\n';
			    (void) strncpy(tmpmessage+j, yypvt[-0].map, i - 1);
			    tmpmessage[j + i - 1] = 0;
			}
			Free(yypvt[-0].map);
		  } break;
case 23:
# line 340 "lev_comp.y"
{
			if(special_lev.nrobjects) {
			    yyerror("Object registers already initialized!");
			} else {
			    special_lev.nrobjects = n_olist;
			    special_lev.robjects = (char *) alloc(n_olist);
			    (void) memcpy((genericptr_t)special_lev.robjects,
					  (genericptr_t)olist, n_olist);
			}
		  } break;
case 24:
# line 351 "lev_comp.y"
{
			if(special_lev.nrmonst) {
			    yyerror("Monster registers already initialized!");
			} else {
			    special_lev.nrmonst = n_mlist;
			    special_lev.rmonst = (char *) alloc(n_mlist);
			    (void) memcpy((genericptr_t)special_lev.rmonst,
					  (genericptr_t)mlist, n_mlist);
			  }
		  } break;
case 25:
# line 364 "lev_comp.y"
{
			tmproom[nrooms] = New(room);
			(void) memset((genericptr_t) tmproom[nrooms], 0,
					sizeof *tmproom[nrooms]);
			tmproom[nrooms]->name = (char *) 0;
			tmproom[nrooms]->parent = (char *) 0;
			tmproom[nrooms]->rtype = 0;
			tmproom[nrooms]->rlit = 0;
			tmproom[nrooms]->xalign = ERR;
			tmproom[nrooms]->yalign = ERR;
			tmproom[nrooms]->x = 0;
			tmproom[nrooms]->y = 0;
			tmproom[nrooms]->w = 2;
			tmproom[nrooms]->h = 2;
			in_room = 1;
		  } break;
case 31:
# line 392 "lev_comp.y"
{
			tmpcor[0] = New(corridor);
			tmpcor[0]->src.room = -1;
			ncorridor = 1;
		  } break;
case 34:
# line 404 "lev_comp.y"
{
			tmpcor[ncorridor] = New(corridor);
			tmpcor[ncorridor]->src.room = yypvt[-2].corpos.room;
			tmpcor[ncorridor]->src.wall = yypvt[-2].corpos.wall;
			tmpcor[ncorridor]->src.door = yypvt[-2].corpos.door;
			tmpcor[ncorridor]->dest.room = yypvt[-0].corpos.room;
			tmpcor[ncorridor]->dest.wall = yypvt[-0].corpos.wall;
			tmpcor[ncorridor]->dest.door = yypvt[-0].corpos.door;
			ncorridor++;
			if (ncorridor >= MAX_OF_TYPE) {
				yyerror("Too many corridors in level!");
				ncorridor--;
			}
		  } break;
case 35:
# line 419 "lev_comp.y"
{
			tmpcor[ncorridor] = New(corridor);
			tmpcor[ncorridor]->src.room = yypvt[-2].corpos.room;
			tmpcor[ncorridor]->src.wall = yypvt[-2].corpos.wall;
			tmpcor[ncorridor]->src.door = yypvt[-2].corpos.door;
			tmpcor[ncorridor]->dest.room = -1;
			tmpcor[ncorridor]->dest.wall = yypvt[-0].i;
			ncorridor++;
			if (ncorridor >= MAX_OF_TYPE) {
				yyerror("Too many corridors in level!");
				ncorridor--;
			}
		  } break;
case 36:
# line 435 "lev_comp.y"
{
			if ((unsigned) yypvt[-5].i >= nrooms)
			    yyerror("Wrong room number!");
			yyval.corpos.room = yypvt[-5].i;
			yyval.corpos.wall = yypvt[-3].i;
			yyval.corpos.door = yypvt[-1].i;
		  } break;
case 37:
# line 445 "lev_comp.y"
{
			store_room();
		  } break;
case 38:
# line 449 "lev_comp.y"
{
			store_room();
		  } break;
case 39:
# line 455 "lev_comp.y"
{
			tmproom[nrooms] = New(room);
			(void) memset((genericptr_t) tmproom[nrooms], 0,
					sizeof *tmproom[nrooms]);
			tmproom[nrooms]->parent = yypvt[-1].map;
			tmproom[nrooms]->name = (char *) 0;
			tmproom[nrooms]->rtype = yypvt[-9].i;
			tmproom[nrooms]->rlit = yypvt[-7].i;
			tmproom[nrooms]->filled = yypvt[-0].i;
			tmproom[nrooms]->xalign = ERR;
			tmproom[nrooms]->yalign = ERR;
			tmproom[nrooms]->x = current_coord.x;
			tmproom[nrooms]->y = current_coord.y;
			tmproom[nrooms]->w = current_size.width;
			tmproom[nrooms]->h = current_size.height;
			in_room = 1;
		  } break;
case 40:
# line 475 "lev_comp.y"
{
			tmproom[nrooms] = New(room);
			(void) memset((genericptr_t) tmproom[nrooms], 0,
					sizeof *tmproom[nrooms]);
			tmproom[nrooms]->name = (char *) 0;
			tmproom[nrooms]->parent = (char *) 0;
			tmproom[nrooms]->rtype = yypvt[-9].i;
			tmproom[nrooms]->rlit = yypvt[-7].i;
			tmproom[nrooms]->filled = yypvt[-0].i;
			tmproom[nrooms]->xalign = current_align.x;
			tmproom[nrooms]->yalign = current_align.y;
			tmproom[nrooms]->x = current_coord.x;
			tmproom[nrooms]->y = current_coord.y;
			tmproom[nrooms]->w = current_size.width;
			tmproom[nrooms]->h = current_size.height;
			in_room = 1;
		  } break;
case 41:
# line 495 "lev_comp.y"
{
			yyval.i = 1;
		  } break;
case 42:
# line 499 "lev_comp.y"
{
			yyval.i = yypvt[-0].i;
		  } break;
case 43:
# line 505 "lev_comp.y"
{
			if ( yypvt[-3].i < 1 || yypvt[-3].i > 5 ||
			    yypvt[-1].i < 1 || yypvt[-1].i > 5 ) {
			    yyerror("Room position should be between 1 & 5!");
			} else {
			    current_coord.x = yypvt[-3].i;
			    current_coord.y = yypvt[-1].i;
			}
		  } break;
case 44:
# line 515 "lev_comp.y"
{
			current_coord.x = current_coord.y = ERR;
		  } break;
case 45:
# line 521 "lev_comp.y"
{
			if ( yypvt[-3].i < 0 || yypvt[-1].i < 0) {
			    yyerror("Invalid subroom position !");
			} else {
			    current_coord.x = yypvt[-3].i;
			    current_coord.y = yypvt[-1].i;
			}
		  } break;
case 46:
# line 530 "lev_comp.y"
{
			current_coord.x = current_coord.y = ERR;
		  } break;
case 47:
# line 536 "lev_comp.y"
{
			current_align.x = yypvt[-3].i;
			current_align.y = yypvt[-1].i;
		  } break;
case 48:
# line 541 "lev_comp.y"
{
			current_align.x = current_align.y = ERR;
		  } break;
case 49:
# line 547 "lev_comp.y"
{
			current_size.width = yypvt[-3].i;
			current_size.height = yypvt[-1].i;
		  } break;
case 50:
# line 552 "lev_comp.y"
{
			current_size.height = current_size.width = ERR;
		  } break;
case 66:
# line 577 "lev_comp.y"
{
			if (tmproom[nrooms]->name)
			    yyerror("This room already has a name!");
			else
			    tmproom[nrooms]->name = yypvt[-0].map;
		  } break;
case 67:
# line 586 "lev_comp.y"
{
			if (tmproom[nrooms]->chance)
			    yyerror("This room already assigned a chance!");
			else if (tmproom[nrooms]->rtype == OROOM)
			    yyerror("Only typed rooms can have a chance!");
			else if (yypvt[-0].i < 1 || yypvt[-0].i > 99)
			    yyerror("The chance is supposed to be precentile.");
			else
			    tmproom[nrooms]->chance = yypvt[-0].i;
		   } break;
case 68:
# line 599 "lev_comp.y"
{
			/* ERR means random here */
			if (yypvt[-2].i == ERR && yypvt[-0].i != ERR) {
		     yyerror("If the door wall is random, so must be its pos!");
			} else {
			    tmprdoor[ndoor] = New(room_door);
			    tmprdoor[ndoor]->secret = yypvt[-6].i;
			    tmprdoor[ndoor]->mask = yypvt[-4].i;
			    tmprdoor[ndoor]->wall = yypvt[-2].i;
			    tmprdoor[ndoor]->pos = yypvt[-0].i;
			    ndoor++;
			    if (ndoor >= MAX_OF_TYPE) {
				    yyerror("Too many doors in room!");
				    ndoor--;
			    }
			}
		  } break;
case 75:
# line 631 "lev_comp.y"
{
			maze.filling = yypvt[-0].i;
			if (index(yypvt[-2].map, '.'))
			    yyerror("Invalid dot ('.') in level name.");
			if ((int) strlen(yypvt[-2].map) > 8)
			    yyerror("Level names limited to 8 characters.");
			yyval.map = yypvt[-2].map;
			in_room = 0;
			n_plist = n_mlist = n_olist = 0;
		  } break;
case 76:
# line 644 "lev_comp.y"
{
			yyval.i = get_floor_type((char)yypvt[-0].i);
		  } break;
case 77:
# line 648 "lev_comp.y"
{
			yyval.i = -1;
		  } break;
case 80:
# line 658 "lev_comp.y"
{
			store_part();
		  } break;
case 81:
# line 664 "lev_comp.y"
{
			tmppart[npart] = New(mazepart);
			tmppart[npart]->halign = 1;
			tmppart[npart]->valign = 1;
			tmppart[npart]->nrobjects = 0;
			tmppart[npart]->nloc = 0;
			tmppart[npart]->nrmonst = 0;
			tmppart[npart]->xsize = 1;
			tmppart[npart]->ysize = 1;
			tmppart[npart]->map = (char **) alloc(sizeof(char *));
			tmppart[npart]->map[0] = (char *) alloc(1);
			tmppart[npart]->map[0][0] = STONE;
			max_x_map = COLNO-1;
			max_y_map = ROWNO;
		  } break;
case 82:
# line 680 "lev_comp.y"
{
			tmppart[npart] = New(mazepart);
			tmppart[npart]->halign = yypvt[-1].i % 10;
			tmppart[npart]->valign = yypvt[-1].i / 10;
			tmppart[npart]->nrobjects = 0;
			tmppart[npart]->nloc = 0;
			tmppart[npart]->nrmonst = 0;
			scan_map(yypvt[-0].map);
			Free(yypvt[-0].map);
		  } break;
case 83:
# line 693 "lev_comp.y"
{
			yyval.i = yypvt[-2].i + (yypvt[-0].i * 10);
		  } break;
case 90:
# line 711 "lev_comp.y"
{
			if (tmppart[npart]->nrobjects) {
			    yyerror("Object registers already initialized!");
			} else {
			    tmppart[npart]->robjects = (char *)alloc(n_olist);
			    (void) memcpy((genericptr_t)tmppart[npart]->robjects,
					  (genericptr_t)olist, n_olist);
			    tmppart[npart]->nrobjects = n_olist;
			}
		  } break;
case 91:
# line 722 "lev_comp.y"
{
			if (tmppart[npart]->nloc) {
			    yyerror("Location registers already initialized!");
			} else {
			    register int i;
			    tmppart[npart]->rloc_x = (char *) alloc(n_plist);
			    tmppart[npart]->rloc_y = (char *) alloc(n_plist);
			    for(i=0;i<n_plist;i++) {
				tmppart[npart]->rloc_x[i] = plist[i].x;
				tmppart[npart]->rloc_y[i] = plist[i].y;
			    }
			    tmppart[npart]->nloc = n_plist;
			}
		  } break;
case 92:
# line 737 "lev_comp.y"
{
			if (tmppart[npart]->nrmonst) {
			    yyerror("Monster registers already initialized!");
			} else {
			    tmppart[npart]->rmonst = (char *) alloc(n_mlist);
			    (void) memcpy((genericptr_t)tmppart[npart]->rmonst,
					  (genericptr_t)mlist, n_mlist);
			    tmppart[npart]->nrmonst = n_mlist;
			}
		  } break;
case 93:
# line 750 "lev_comp.y"
{
			if (n_olist < MAX_REGISTERS)
			    olist[n_olist++] = yypvt[-0].i;
			else
			    yyerror("Object list too long!");
		  } break;
case 94:
# line 757 "lev_comp.y"
{
			if (n_olist < MAX_REGISTERS)
			    olist[n_olist++] = yypvt[-2].i;
			else
			    yyerror("Object list too long!");
		  } break;
case 95:
# line 766 "lev_comp.y"
{
			if (n_mlist < MAX_REGISTERS)
			    mlist[n_mlist++] = yypvt[-0].i;
			else
			    yyerror("Monster list too long!");
		  } break;
case 96:
# line 773 "lev_comp.y"
{
			if (n_mlist < MAX_REGISTERS)
			    mlist[n_mlist++] = yypvt[-2].i;
			else
			    yyerror("Monster list too long!");
		  } break;
case 97:
# line 782 "lev_comp.y"
{
			if (n_plist < MAX_REGISTERS)
			    plist[n_plist++] = current_coord;
			else
			    yyerror("Location list too long!");
		  } break;
case 98:
# line 789 "lev_comp.y"
{
			if (n_plist < MAX_REGISTERS)
			    plist[n_plist++] = current_coord;
			else
			    yyerror("Location list too long!");
		  } break;
case 122:
# line 825 "lev_comp.y"
{
			tmpmonst[nmons] = New(monster);
			tmpmonst[nmons]->x = current_coord.x;
			tmpmonst[nmons]->y = current_coord.y;
			tmpmonst[nmons]->class = yypvt[-4].i;
			tmpmonst[nmons]->peaceful = -1; /* no override */
			tmpmonst[nmons]->asleep = -1;
			tmpmonst[nmons]->align = - MAX_REGISTERS - 2;
			tmpmonst[nmons]->name.str = 0;
			tmpmonst[nmons]->appear = 0;
			tmpmonst[nmons]->appear_as.str = 0;
			if (!in_room)
			    check_coord(current_coord.x, current_coord.y,
					"Monster");
			if (!yypvt[-2].map)
			    tmpmonst[nmons]->id = NON_PM;
			else {
				int token = get_monster_id(yypvt[-2].map, (char) yypvt[-4].i);
				if (token == ERR) {
				    yywarning(
			      "Invalid monster name!  Making random monster.");
				    tmpmonst[nmons]->id = NON_PM;
				} else
				    tmpmonst[nmons]->id = token;
				Free(yypvt[-2].map);
			}
		  } break;
case 123:
# line 853 "lev_comp.y"
{
			nmons++;
			if (nmons >= MAX_OF_TYPE) {
			    yyerror("Too many monsters in room or mazepart!");
			    nmons--;
			}
		  } break;
case 126:
# line 867 "lev_comp.y"
{
			tmpmonst[nmons]->name.str = yypvt[-0].map;
		  } break;
case 127:
# line 871 "lev_comp.y"
{
			tmpmonst[nmons]->peaceful = yypvt[-0].i;
		  } break;
case 128:
# line 875 "lev_comp.y"
{
			tmpmonst[nmons]->asleep = yypvt[-0].i;
		  } break;
case 129:
# line 879 "lev_comp.y"
{
			tmpmonst[nmons]->align = yypvt[-0].i;
		  } break;
case 130:
# line 883 "lev_comp.y"
{
			tmpmonst[nmons]->appear = yypvt[-1].i;
			tmpmonst[nmons]->appear_as.str = yypvt[-0].map;
		  } break;
case 131:
# line 890 "lev_comp.y"
{
		  } break;
case 132:
# line 893 "lev_comp.y"
{
			/* 1: is contents of next object with 2 */
			/* 2: is a container */
			/* 0: neither */
			tmpobj[nobj-1]->containment = 2;
		  } break;
case 133:
# line 902 "lev_comp.y"
{
			tmpobj[nobj] = New(object);
			tmpobj[nobj]->class = yypvt[-2].i;
			tmpobj[nobj]->corpsenm = NON_PM;
			tmpobj[nobj]->curse_state = -1;
			tmpobj[nobj]->name.str = 0;
			if (!yypvt[-0].map)
			    tmpobj[nobj]->id = -1;
			else {
				int token = get_object_id(yypvt[-0].map);
				if (token == ERR) {
				    yywarning("Illegal object name!  Making random object.");
				    tmpobj[nobj]->id = -1;
				} else
				    tmpobj[nobj]->id = token;
				Free(yypvt[-0].map);
			}
		  } break;
case 134:
# line 921 "lev_comp.y"
{
			nobj++;
			if (nobj >= MAX_OF_TYPE) {
				yyerror("Too many objects in room or mazepart!");
				nobj--;
			}
		  } break;
case 135:
# line 931 "lev_comp.y"
{
			tmpobj[nobj]->containment = 0;
			tmpobj[nobj]->x = current_coord.x;
			tmpobj[nobj]->y = current_coord.y;
			if (!in_room)
			    check_coord(current_coord.x, current_coord.y,
					"Object");
		  } break;
case 136:
# line 940 "lev_comp.y"
{
			tmpobj[nobj]->containment = 1;
			/* random coordinate, will be overridden anyway */
			tmpobj[nobj]->x = -MAX_REGISTERS-1;
			tmpobj[nobj]->y = -MAX_REGISTERS-1;
		  } break;
case 137:
# line 949 "lev_comp.y"
{
			tmpobj[nobj]->spe = -127;
	/* Note below: we're trying to make as many of these optional as
	 * possible.  We clearly can't make curse_state, enchantment, and
	 * monster_id _all_ optional, since ",random" would be ambiguous.
	 * We can't even just make enchantment mandatory, since if we do that
	 * alone, ",random" requires too much lookahead to parse.
	 */
		  } break;
case 138:
# line 959 "lev_comp.y"
{
		  } break;
case 139:
# line 962 "lev_comp.y"
{
		  } break;
case 140:
# line 965 "lev_comp.y"
{
		  } break;
case 141:
# line 970 "lev_comp.y"
{
			tmpobj[nobj]->curse_state = -1;
		  } break;
case 142:
# line 974 "lev_comp.y"
{
			tmpobj[nobj]->curse_state = yypvt[-0].i;
		  } break;
case 143:
# line 980 "lev_comp.y"
{
			int token = get_monster_id(yypvt[-0].map, (char)0);
			if (token == ERR)	/* "random" */
			    tmpobj[nobj]->corpsenm = NON_PM - 1;
			else
			    tmpobj[nobj]->corpsenm = token;
			Free(yypvt[-0].map);
		  } break;
case 144:
# line 991 "lev_comp.y"
{
			tmpobj[nobj]->spe = -127;
		  } break;
case 145:
# line 995 "lev_comp.y"
{
			tmpobj[nobj]->spe = yypvt[-0].i;
		  } break;
case 147:
# line 1002 "lev_comp.y"
{
		  } break;
case 148:
# line 1005 "lev_comp.y"
{
			tmpobj[nobj]->name.str = yypvt[-0].map;
		  } break;
case 149:
# line 1011 "lev_comp.y"
{
			tmpdoor[ndoor] = New(door);
			tmpdoor[ndoor]->x = current_coord.x;
			tmpdoor[ndoor]->y = current_coord.y;
			tmpdoor[ndoor]->mask = yypvt[-2].i;
			if(current_coord.x >= 0 && current_coord.y >= 0 &&
			   tmpmap[current_coord.y][current_coord.x] != DOOR &&
			   tmpmap[current_coord.y][current_coord.x] != SDOOR)
			    yyerror("Door decl doesn't match the map");
			ndoor++;
			if (ndoor >= MAX_OF_TYPE) {
				yyerror("Too many doors in mazepart!");
				ndoor--;
			}
		  } break;
case 150:
# line 1029 "lev_comp.y"
{
			tmptrap[ntrap] = New(trap);
			tmptrap[ntrap]->x = current_coord.x;
			tmptrap[ntrap]->y = current_coord.y;
			tmptrap[ntrap]->type = yypvt[-2].i;
			tmptrap[ntrap]->chance = 100;
			if (!in_room)
			    check_coord(current_coord.x, current_coord.y,
					"Trap");
			ntrap++;
			if (ntrap >= MAX_OF_TYPE) {
				yyerror("Too many traps in room or mazepart!");
				ntrap--;
			}
		  } break;
case 151:
# line 1045 "lev_comp.y"
{
			tmptrap[ntrap] = New(trap);
			tmptrap[ntrap]->x = current_coord.x;
			tmptrap[ntrap]->y = current_coord.y;
			tmptrap[ntrap]->type = yypvt[-4].i;
			tmptrap[ntrap]->chance = yypvt[-0].i;
			if (!in_room)
			    check_coord(current_coord.x, current_coord.y,
					"Trap");
			ntrap++;
			if (ntrap >= MAX_OF_TYPE) {
				yyerror("Too many traps in room or mazepart!");
				ntrap--;
			}
		  } break;
case 152:
# line 1063 "lev_comp.y"
{
		        int x, y, dir;

			tmpdb[ndb] = New(drawbridge);
			x = tmpdb[ndb]->x = current_coord.x;
			y = tmpdb[ndb]->y = current_coord.y;
			/* convert dir from a DIRECTION to a DB_DIR */
			dir = yypvt[-2].i;
			switch(dir) {
			case W_NORTH: dir = DB_NORTH; y--; break;
			case W_SOUTH: dir = DB_SOUTH; y++; break;
			case W_EAST:  dir = DB_EAST;  x++; break;
			case W_WEST:  dir = DB_WEST;  x--; break;
			default:
			    yyerror("Invalid drawbridge direction");
			    break;
			}
			tmpdb[ndb]->dir = dir;
			if (current_coord.x >= 0 && current_coord.y >= 0 &&
			    !IS_WALL(tmpmap[y][x])) {
			    char ebuf[60];
			    Sprintf(ebuf,
				    "Wall needed for drawbridge (%02d, %02d)",
				    current_coord.x, current_coord.y);
			    yyerror(ebuf);
			}

			if ( yypvt[-0].i == D_ISOPEN )
			    tmpdb[ndb]->db_open = 1;
			else if ( yypvt[-0].i == D_CLOSED )
			    tmpdb[ndb]->db_open = 0;
			else
			    yyerror("A drawbridge can only be open or closed!");
			ndb++;
			if (ndb >= MAX_OF_TYPE) {
				yyerror("Too many drawbridges in mazepart!");
				ndb--;
			}
		   } break;
case 153:
# line 1105 "lev_comp.y"
{
			tmpwalk[nwalk] = New(walk);
			tmpwalk[nwalk]->x = current_coord.x;
			tmpwalk[nwalk]->y = current_coord.y;
			tmpwalk[nwalk]->dir = yypvt[-0].i;
			nwalk++;
			if (nwalk >= MAX_OF_TYPE) {
				yyerror("Too many mazewalks in mazepart!");
				nwalk--;
			}
		  } break;
case 154:
# line 1119 "lev_comp.y"
{
			wallify_map();
		  } break;
case 155:
# line 1125 "lev_comp.y"
{
			tmplad[nlad] = New(lad);
			tmplad[nlad]->x = current_coord.x;
			tmplad[nlad]->y = current_coord.y;
			tmplad[nlad]->up = yypvt[-0].i;
			if (!in_room)
			    check_coord(current_coord.x, current_coord.y,
					"Ladder");
			nlad++;
			if (nlad >= MAX_OF_TYPE) {
				yyerror("Too many ladders in mazepart!");
				nlad--;
			}
		  } break;
case 156:
# line 1142 "lev_comp.y"
{
			tmpstair[nstair] = New(stair);
			tmpstair[nstair]->x = current_coord.x;
			tmpstair[nstair]->y = current_coord.y;
			tmpstair[nstair]->up = yypvt[-0].i;
			if (!in_room)
			    check_coord(current_coord.x, current_coord.y,
					"Stairway");
			nstair++;
			if (nstair >= MAX_OF_TYPE) {
				yyerror("Too many stairs in room or mazepart!");
				nstair--;
			}
		  } break;
case 157:
# line 1159 "lev_comp.y"
{
			tmplreg[nlreg] = New(lev_region);
			tmplreg[nlreg]->in_islev = yypvt[-0].i;
			tmplreg[nlreg]->inarea.x1 = current_region.x1;
			tmplreg[nlreg]->inarea.y1 = current_region.y1;
			tmplreg[nlreg]->inarea.x2 = current_region.x2;
			tmplreg[nlreg]->inarea.y2 = current_region.y2;
		  } break;
case 158:
# line 1168 "lev_comp.y"
{
			tmplreg[nlreg]->del_islev = yypvt[-2].i;
			tmplreg[nlreg]->delarea.x1 = current_region.x1;
			tmplreg[nlreg]->delarea.y1 = current_region.y1;
			tmplreg[nlreg]->delarea.x2 = current_region.x2;
			tmplreg[nlreg]->delarea.y2 = current_region.y2;
			if(yypvt[-0].i)
			    tmplreg[nlreg]->rtype = LR_UPSTAIR;
			else
			    tmplreg[nlreg]->rtype = LR_DOWNSTAIR;
			tmplreg[nlreg]->rname.str = 0;
			nlreg++;
			if (nlreg >= MAX_OF_TYPE) {
				yyerror("Too many levregions in mazepart!");
				nlreg--;
			}
		  } break;
case 159:
# line 1188 "lev_comp.y"
{
			tmplreg[nlreg] = New(lev_region);
			tmplreg[nlreg]->in_islev = yypvt[-0].i;
			tmplreg[nlreg]->inarea.x1 = current_region.x1;
			tmplreg[nlreg]->inarea.y1 = current_region.y1;
			tmplreg[nlreg]->inarea.x2 = current_region.x2;
			tmplreg[nlreg]->inarea.y2 = current_region.y2;
		  } break;
case 160:
# line 1197 "lev_comp.y"
{
			tmplreg[nlreg]->del_islev = yypvt[-2].i;
			tmplreg[nlreg]->delarea.x1 = current_region.x1;
			tmplreg[nlreg]->delarea.y1 = current_region.y1;
			tmplreg[nlreg]->delarea.x2 = current_region.x2;
			tmplreg[nlreg]->delarea.y2 = current_region.y2;
			tmplreg[nlreg]->rtype = LR_PORTAL;
			tmplreg[nlreg]->rname.str = yypvt[-0].map;
			nlreg++;
			if (nlreg >= MAX_OF_TYPE) {
				yyerror("Too many levregions in mazepart!");
				nlreg--;
			}
		  } break;
case 161:
# line 1214 "lev_comp.y"
{
			tmplreg[nlreg] = New(lev_region);
			tmplreg[nlreg]->in_islev = yypvt[-0].i;
			tmplreg[nlreg]->inarea.x1 = current_region.x1;
			tmplreg[nlreg]->inarea.y1 = current_region.y1;
			tmplreg[nlreg]->inarea.x2 = current_region.x2;
			tmplreg[nlreg]->inarea.y2 = current_region.y2;
		  } break;
case 162:
# line 1223 "lev_comp.y"
{
			tmplreg[nlreg]->del_islev = yypvt[-0].i;
			tmplreg[nlreg]->delarea.x1 = current_region.x1;
			tmplreg[nlreg]->delarea.y1 = current_region.y1;
			tmplreg[nlreg]->delarea.x2 = current_region.x2;
			tmplreg[nlreg]->delarea.y2 = current_region.y2;
		  } break;
case 163:
# line 1231 "lev_comp.y"
{
			switch(yypvt[-0].i) {
			case -1: tmplreg[nlreg]->rtype = LR_TELE; break;
			case 0: tmplreg[nlreg]->rtype = LR_DOWNTELE; break;
			case 1: tmplreg[nlreg]->rtype = LR_UPTELE; break;
			}
			tmplreg[nlreg]->rname.str = 0;
			nlreg++;
			if (nlreg >= MAX_OF_TYPE) {
				yyerror("Too many levregions in mazepart!");
				nlreg--;
			}
		  } break;
case 164:
# line 1247 "lev_comp.y"
{
			tmplreg[nlreg] = New(lev_region);
			tmplreg[nlreg]->in_islev = yypvt[-0].i;
			tmplreg[nlreg]->inarea.x1 = current_region.x1;
			tmplreg[nlreg]->inarea.y1 = current_region.y1;
			tmplreg[nlreg]->inarea.x2 = current_region.x2;
			tmplreg[nlreg]->inarea.y2 = current_region.y2;
		  } break;
case 165:
# line 1256 "lev_comp.y"
{
			tmplreg[nlreg]->del_islev = yypvt[-0].i;
			tmplreg[nlreg]->delarea.x1 = current_region.x1;
			tmplreg[nlreg]->delarea.y1 = current_region.y1;
			tmplreg[nlreg]->delarea.x2 = current_region.x2;
			tmplreg[nlreg]->delarea.y2 = current_region.y2;
			tmplreg[nlreg]->rtype = LR_BRANCH;
			tmplreg[nlreg]->rname.str = 0;
			nlreg++;
			if (nlreg >= MAX_OF_TYPE) {
				yyerror("Too many levregions in mazepart!");
				nlreg--;
			}
		  } break;
case 166:
# line 1273 "lev_comp.y"
{
			yyval.i = -1;
		  } break;
case 167:
# line 1277 "lev_comp.y"
{
			yyval.i = yypvt[-0].i;
		  } break;
case 168:
# line 1283 "lev_comp.y"
{
			yyval.i = 0;
		  } break;
case 169:
# line 1287 "lev_comp.y"
{
/* This series of if statements is a hack for MSC 5.1.  It seems that its
   tiny little brain cannot compile if these are all one big if statement. */
			if (yypvt[-7].i <= 0 || yypvt[-7].i >= COLNO)
				yyerror("Region out of level range!");
			else if (yypvt[-5].i < 0 || yypvt[-5].i >= ROWNO)
				yyerror("Region out of level range!");
			else if (yypvt[-3].i <= 0 || yypvt[-3].i >= COLNO)
				yyerror("Region out of level range!");
			else if (yypvt[-1].i < 0 || yypvt[-1].i >= ROWNO)
				yyerror("Region out of level range!");
			current_region.x1 = yypvt[-7].i;
			current_region.y1 = yypvt[-5].i;
			current_region.x2 = yypvt[-3].i;
			current_region.y2 = yypvt[-1].i;
			yyval.i = 1;
		  } break;
case 170:
# line 1307 "lev_comp.y"
{
			tmpfountain[nfountain] = New(fountain);
			tmpfountain[nfountain]->x = current_coord.x;
			tmpfountain[nfountain]->y = current_coord.y;
			if (!in_room)
			    check_coord(current_coord.x, current_coord.y,
					"Fountain");
			nfountain++;
			if (nfountain >= MAX_OF_TYPE) {
			    yyerror("Too many fountains in room or mazepart!");
			    nfountain--;
			}
		  } break;
case 171:
# line 1323 "lev_comp.y"
{
			tmpsink[nsink] = New(sink);
			tmpsink[nsink]->x = current_coord.x;
			tmpsink[nsink]->y = current_coord.y;
			nsink++;
			if (nsink >= MAX_OF_TYPE) {
				yyerror("Too many sinks in room!");
				nsink--;
			}
		  } break;
case 172:
# line 1336 "lev_comp.y"
{
			tmppool[npool] = New(pool);
			tmppool[npool]->x = current_coord.x;
			tmppool[npool]->y = current_coord.y;
			npool++;
			if (npool >= MAX_OF_TYPE) {
				yyerror("Too many pools in room!");
				npool--;
			}
		  } break;
case 173:
# line 1349 "lev_comp.y"
{
			tmpdig[ndig] = New(digpos);
			tmpdig[ndig]->x1 = current_region.x1;
			tmpdig[ndig]->y1 = current_region.y1;
			tmpdig[ndig]->x2 = current_region.x2;
			tmpdig[ndig]->y2 = current_region.y2;
			ndig++;
			if (ndig >= MAX_OF_TYPE) {
				yyerror("Too many diggables in mazepart!");
				ndig--;
			}
		  } break;
case 174:
# line 1364 "lev_comp.y"
{
			tmppass[npass] = New(digpos);
			tmppass[npass]->x1 = current_region.x1;
			tmppass[npass]->y1 = current_region.y1;
			tmppass[npass]->x2 = current_region.x2;
			tmppass[npass]->y2 = current_region.y2;
			npass++;
			if (npass >= 32) {
				yyerror("Too many passwalls in mazepart!");
				npass--;
			}
		  } break;
case 175:
# line 1379 "lev_comp.y"
{
			tmpreg[nreg] = New(region);
			tmpreg[nreg]->x1 = current_region.x1;
			tmpreg[nreg]->y1 = current_region.y1;
			tmpreg[nreg]->x2 = current_region.x2;
			tmpreg[nreg]->y2 = current_region.y2;
			tmpreg[nreg]->rlit = yypvt[-3].i;
			tmpreg[nreg]->rtype = yypvt[-1].i;
			if(yypvt[-0].i & 1) tmpreg[nreg]->rtype += MAXRTYPE+1;
			tmpreg[nreg]->rirreg = ((yypvt[-0].i & 2) != 0);
			if(current_region.x1 > current_region.x2 ||
			   current_region.y1 > current_region.y2)
			   yyerror("Region start > end!");
			if(tmpreg[nreg]->rtype == VAULT &&
			   (tmpreg[nreg]->rirreg ||
			    (tmpreg[nreg]->x2 - tmpreg[nreg]->x1 != 1) ||
			    (tmpreg[nreg]->y2 - tmpreg[nreg]->y1 != 1)))
				yyerror("Vaults must be exactly 2x2!");
			if(want_warnings && !tmpreg[nreg]->rirreg &&
			   current_region.x1 > 0 && current_region.y1 > 0 &&
			   current_region.x2 < (int)max_x_map &&
			   current_region.y2 < (int)max_y_map) {
			    /* check for walls in the room */
			    char ebuf[60];
			    register int x, y, nrock = 0;

			    for(y=current_region.y1; y<=current_region.y2; y++)
				for(x=current_region.x1;
				    x<=current_region.x2; x++)
				    if(IS_ROCK(tmpmap[y][x]) ||
				       IS_DOOR(tmpmap[y][x])) nrock++;
			    if(nrock) {
				Sprintf(ebuf,
					"Rock in room (%02d,%02d,%02d,%02d)?!",
					current_region.x1, current_region.y1,
					current_region.x2, current_region.y2);
				yywarning(ebuf);
			    }
			    if (
		!IS_ROCK(tmpmap[current_region.y1-1][current_region.x1-1]) ||
		!IS_ROCK(tmpmap[current_region.y2+1][current_region.x1-1]) ||
		!IS_ROCK(tmpmap[current_region.y1-1][current_region.x2+1]) ||
		!IS_ROCK(tmpmap[current_region.y2+1][current_region.x2+1])) {
				Sprintf(ebuf,
				"NonRock edge in room (%02d,%02d,%02d,%02d)?!",
					current_region.x1, current_region.y1,
					current_region.x2, current_region.y2);
				yywarning(ebuf);
			    }
			} else if(tmpreg[nreg]->rirreg &&
		!IS_ROOM(tmpmap[current_region.y1][current_region.x1])) {
			    char ebuf[60];
			    Sprintf(ebuf,
				    "Rock in irregular room (%02d,%02d)?!",
				    current_region.x1, current_region.y1);
			    yyerror(ebuf);
			}
			nreg++;
			if (nreg >= MAX_OF_TYPE) {
				yyerror("Too many regions in mazepart!");
				nreg--;
			}
		  } break;
case 176:
# line 1445 "lev_comp.y"
{
			tmpaltar[naltar] = New(altar);
			tmpaltar[naltar]->x = current_coord.x;
			tmpaltar[naltar]->y = current_coord.y;
			tmpaltar[naltar]->align = yypvt[-2].i;
			tmpaltar[naltar]->shrine = yypvt[-0].i;
			if (!in_room)
			    check_coord(current_coord.x, current_coord.y,
					"Altar");
			naltar++;
			if (naltar >= MAX_OF_TYPE) {
				yyerror("Too many altars in room or mazepart!");
				naltar--;
			}
		  } break;
case 177:
# line 1463 "lev_comp.y"
{
			tmpgold[ngold] = New(gold);
			tmpgold[ngold]->x = current_coord.x;
			tmpgold[ngold]->y = current_coord.y;
			tmpgold[ngold]->amount = yypvt[-2].i;
			if (!in_room)
			    check_coord(current_coord.x, current_coord.y,
					"Gold");
			ngold++;
			if (ngold >= MAX_OF_TYPE) {
				yyerror("Too many golds in room or mazepart!");
				ngold--;
			}
		  } break;
case 178:
# line 1480 "lev_comp.y"
{
			tmpengraving[nengraving] = New(engraving);
			tmpengraving[nengraving]->x = current_coord.x;
			tmpengraving[nengraving]->y = current_coord.y;
			tmpengraving[nengraving]->engr.str = yypvt[-0].map;
			tmpengraving[nengraving]->etype = yypvt[-2].i;
			if (!in_room)
			    check_coord(current_coord.x, current_coord.y,
					"Engraving");
			nengraving++;
			if (nengraving >= MAX_OF_TYPE) {
			    yyerror("Too many engravings in room or mazepart!");
			    nengraving--;
			}
		  } break;
case 180:
# line 1499 "lev_comp.y"
{
			yyval.i = - MAX_REGISTERS - 1;
		  } break;
case 183:
# line 1507 "lev_comp.y"
{
			yyval.i = - MAX_REGISTERS - 1;
		  } break;
case 186:
# line 1515 "lev_comp.y"
{
			yyval.map = (char *) 0;
		  } break;
case 188:
# line 1522 "lev_comp.y"
{
			yyval.map = (char *) 0;
		  } break;
case 189:
# line 1528 "lev_comp.y"
{
			int token = get_trap_type(yypvt[-0].map);
			if (token == ERR)
				yyerror("Unknown trap type!");
			yyval.i = token;
			Free(yypvt[-0].map);
		  } break;
case 191:
# line 1539 "lev_comp.y"
{
			if (tmptrap[ntrap]->chance)
			    yyerror("This trap already assigned a chance!");
			else if (yypvt[-0].i < 1 || yypvt[-0].i > 99)
			    yyerror("The chance is supposed to be percentile.");
			else
			    tmptrap[ntrap]->chance = yypvt[-0].i;
		   } break;
case 192:
# line 1550 "lev_comp.y"
{
			int token = get_room_type(yypvt[-0].map);
			if (token == ERR) {
				yywarning("Unknown room type!  Making ordinary room...");
				yyval.i = OROOM;
			} else
				yyval.i = token;
			Free(yypvt[-0].map);
		  } break;
case 194:
# line 1563 "lev_comp.y"
{
			yyval.i = 0;
		  } break;
case 195:
# line 1567 "lev_comp.y"
{
			yyval.i = yypvt[-0].i;
		  } break;
case 196:
# line 1571 "lev_comp.y"
{
			yyval.i = yypvt[-2].i + (yypvt[-0].i << 1);
		  } break;
case 199:
# line 1579 "lev_comp.y"
{
			current_coord.x = current_coord.y = -MAX_REGISTERS-1;
		  } break;
case 206:
# line 1595 "lev_comp.y"
{
			yyval.i = - MAX_REGISTERS - 1;
		  } break;
case 209:
# line 1605 "lev_comp.y"
{
			if ( yypvt[-1].i >= MAX_REGISTERS )
				yyerror("Register Index overflow!");
			else
				current_coord.x = current_coord.y = - yypvt[-1].i - 1;
		  } break;
case 210:
# line 1614 "lev_comp.y"
{
			if ( yypvt[-1].i >= MAX_REGISTERS )
				yyerror("Register Index overflow!");
			else
				yyval.i = - yypvt[-1].i - 1;
		  } break;
case 211:
# line 1623 "lev_comp.y"
{
			if ( yypvt[-1].i >= MAX_REGISTERS )
				yyerror("Register Index overflow!");
			else
				yyval.i = - yypvt[-1].i - 1;
		  } break;
case 212:
# line 1632 "lev_comp.y"
{
			if ( yypvt[-1].i >= 3 )
				yyerror("Register Index overflow!");
			else
				yyval.i = - yypvt[-1].i - 1;
		  } break;
case 214:
# line 1644 "lev_comp.y"
{
			if (check_monster_char((char) yypvt[-0].i))
				yyval.i = yypvt[-0].i ;
			else {
				yyerror("Unknown monster class!");
				yyval.i = ERR;
			}
		  } break;
case 215:
# line 1655 "lev_comp.y"
{
			char c = yypvt[-0].i;
			if (check_object_char(c))
				yyval.i = c;
			else {
				yyerror("Unknown char class!");
				yyval.i = ERR;
			}
		  } break;
case 221:
# line 1678 "lev_comp.y"
{
			if (!in_room && !init_lev.init_present &&
			    (yypvt[-3].i < 0 || yypvt[-3].i > (int)max_x_map ||
			     yypvt[-1].i < 0 || yypvt[-1].i > (int)max_y_map))
			    yyerror("Coordinates out of map range!");
			current_coord.x = yypvt[-3].i;
			current_coord.y = yypvt[-1].i;
		  } break;
case 222:
# line 1689 "lev_comp.y"
{
/* This series of if statements is a hack for MSC 5.1.  It seems that its
   tiny little brain cannot compile if these are all one big if statement. */
			if (yypvt[-7].i < 0 || yypvt[-7].i > (int)max_x_map)
				yyerror("Region out of map range!");
			else if (yypvt[-5].i < 0 || yypvt[-5].i > (int)max_y_map)
				yyerror("Region out of map range!");
			else if (yypvt[-3].i < 0 || yypvt[-3].i > (int)max_x_map)
				yyerror("Region out of map range!");
			else if (yypvt[-1].i < 0 || yypvt[-1].i > (int)max_y_map)
				yyerror("Region out of map range!");
			current_region.x1 = yypvt[-7].i;
			current_region.y1 = yypvt[-5].i;
			current_region.x2 = yypvt[-3].i;
			current_region.y2 = yypvt[-1].i;
		  } break;
	}
	goto yystack;		/* reset registers in driver code */
}

#undef YYCONST


/*
 * In SCO C++, crt1.o calls _main() (in libC.a) which calls main().
 * If we define a main() in liby.a, it will not get extracted, since
 * liby.a appears before libC.a on the link line.  To get around this
 * the linker must know we need main() when it is processing liby.a
 * So, we have a dummy call right here.
*/
#if defined(__cplusplus)
extern "C" int main(void);
void _yydumMain_() { main(); }
#endif

